#!/usr/bin/python
#
# Copyright (c) 2021 Aparna Patil(@techcon65)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_proximityplacementgroup

version_added: "1.6.0"

short_description: Create, delete and update proximity placement group

description:
    - Creates, deletes, and updates proximity placement group.

options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    name:
        description:
            - The name of the proximity placement group.
        required: true
        type: str
    location:
        description:
            - Valid Azure location for proximity placement group. Defaults to location of resource group.
        type: str
    state:
        description:
            - Assert the state of the placement group. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Aparna Patil (@techcon65)
'''

EXAMPLES = '''
- name: Create a proximity placement group
  azure_rm_proximityplacementgroup:
    resource_group: myAzureResourceGroup
    location: eastus
    name: myppg
    state: present

- name: Update proximity placement group
  azure_rm_proximityplacementgroup:
    resource_group: myAzureResourceGroup
    location: eastus
    name: myppg
    tags:
      key1: "value1"
    state: present

- name: Delete a proximity placement group
  azure_rm_proximityplacementgroup:
    resource_group: myAzureResourceGroup
    name: myppg
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the proximity placement group.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The proximity placement group ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/providers/
                     Microsoft.Compute/proximityPlacementGroups/myppg"
        name:
            description:
                - The proximity placement group name.
            returned: always
            type: str
            sample: 'myppg'
        location:
            description:
                - The Azure Region where the resource lives.
            returned: always
            type: str
            sample: eastus
        proximity_placement_group_type:
            description:
                - The type of proximity placement group.
            returned: always
            type: str
            sample: Standard
        tags:
            description:
                - Resource tags.
            returned: always
            type: list
            sample: [{"key1": "value1"}]
        type:
            description:
                - The type of resource.
            returned: always
            type: str
            sample: Microsoft.Compute/proximityPlacementGroups
'''

from ansible.module_utils.basic import _load_params
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, normalize_location_name

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMProximityPlacementGroup(AzureRMModuleBase):

    def __init__(self):

        _load_params()
        # define user inputs from playbook
        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            location=dict(type='str'),
            state=dict(choices=['present', 'absent'], default='present', type='str')
        )

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.tags = None

        super(AzureRMProximityPlacementGroup, self).__init__(self.module_arg_spec,
                                                             supports_check_mode=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False
        results = dict()
        proximity_placement_group = None

        # retrieve resource group to make sure it exists
        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            # Set default location
            self.location = resource_group.location

        self.location = normalize_location_name(self.location)

        try:
            self.log('Fetching Proximity placement group {0}'.format(self.name))
            proximity_placement_group = self.compute_client.proximity_placement_groups.get(self.resource_group,
                                                                                           self.name)
            # serialize object into a dictionary
            results = self.ppg_to_dict(proximity_placement_group)
            if self.state == 'present':
                changed = False
                update_tags, results['tags'] = self.update_tags(results['tags'])
                if update_tags:
                    changed = True
                self.tags = results['tags']
            elif self.state == 'absent':
                changed = True

        except ResourceNotFoundError:
            if self.state == 'present':
                changed = True
            else:
                changed = False

        self.results['changed'] = changed
        self.results['state'] = results

        if self.check_mode:
            return self.results

        if changed:
            if self.state == 'present':
                # create or update proximity placement group
                proximity_placement_group_new = \
                    self.compute_models.ProximityPlacementGroup(location=self.location,
                                                                proximity_placement_group_type='Standard')
                if self.tags:
                    proximity_placement_group_new.tags = self.tags
                self.results['state'] = self.create_or_update_placementgroup(proximity_placement_group_new)

            elif self.state == 'absent':
                # delete proximity placement group
                self.delete_placementgroup()
                self.results['state'] = 'Deleted'

        return self.results

    def create_or_update_placementgroup(self, proximity_placement_group):
        try:
            # create the placement group
            response = self.compute_client.proximity_placement_groups.create_or_update(resource_group_name=self.resource_group,
                                                                                       proximity_placement_group_name=self.name,
                                                                                       parameters=proximity_placement_group)
        except Exception as exc:
            self.fail("Error creating or updating proximity placement group {0} - {1}".format(self.name, str(exc)))
        return self.ppg_to_dict(response)

    def delete_placementgroup(self):
        try:
            # delete the placement group
            response = self.compute_client.proximity_placement_groups.delete(resource_group_name=self.resource_group,
                                                                             proximity_placement_group_name=self.name)
        except Exception as exc:
            self.fail("Error deleting proximity placement group {0} - {1}".format(self.name, str(exc)))
        return response

    def ppg_to_dict(self, proximityplacementgroup):
        result = proximityplacementgroup.as_dict()
        result['tags'] = proximityplacementgroup.tags
        return result


def main():
    AzureRMProximityPlacementGroup()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_wireless_ssids
short_description: Resource module for networks _wireless _ssids
description:
- Manage operation update of the resource networks _wireless _ssids.
- Update the attributes of an MR SSID.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  activeDirectory:
    description: The current setting for Active Directory. Only valid if splashPage
      is 'Password-protected with Active Directory'.
    suboptions:
      credentials:
        description: (Optional) The credentials of the user account to be used by the
          AP to bind to your Active Directory server. The Active Directory account should
          have permissions on all your Active Directory servers. Only valid if the splashPage
          is 'Password-protected with Active Directory'.
        suboptions:
          logonName:
            description: The logon name of the Active Directory account.
            type: str
          password:
            description: The password to the Active Directory user account.
            type: str
        type: dict
      servers:
        description: The Active Directory servers to be used for authentication.
        elements: dict
        suboptions:
          host:
            description: IP address of your Active Directory server.
            type: str
          port:
            description: (Optional) UDP port the Active Directory server listens on.
              By default, uses port 3268.
            type: int
        type: list
    type: dict
  adultContentFilteringEnabled:
    description: Boolean indicating whether or not adult content will be blocked.
    type: bool
  apTagsAndVlanIds:
    description: The list of tags and VLAN IDs used for VLAN tagging. This param is
      only valid when the ipAssignmentMode is 'Bridge mode' or 'Layer 3 roaming'.
    elements: dict
    suboptions:
      tags:
        description: Array of AP tags.
        elements: str
        type: list
      vlanId:
        description: Numerical identifier that is assigned to the VLAN.
        type: int
    type: list
  authMode:
    description: The association control method for the SSID ('open', 'open-enhanced',
      'psk', 'open-with-radius', 'open-with-nac', '8021x-meraki', '8021x-nac', '8021x-radius',
      '8021x-google', '8021x-localradius', 'ipsk-with-radius' or 'ipsk-without-radius').
    type: str
  availabilityTags:
    description: Accepts a list of tags for this SSID. If availableOnAllAps is false,
      then the SSID will only be broadcast by APs with tags matching any of the tags
      in this list.
    elements: str
    type: list
  availableOnAllAps:
    description: Boolean indicating whether all APs should broadcast the SSID or if
      it should be restricted to APs matching any availability tags. Can only be false
      if the SSID has availability tags.
    type: bool
  bandSelection:
    description: The client-serving radio frequencies of this SSID in the default indoor
      RF profile. ('Dual band operation', '5 GHz band only' or 'Dual band operation
      with Band Steering').
    type: str
  concentratorNetworkId:
    description: The concentrator to use when the ipAssignmentMode is 'Layer 3 roaming
      with a concentrator' or 'VPN'.
    type: str
  defaultVlanId:
    description: The default VLAN ID used for 'all other APs'. This param is only valid
      when the ipAssignmentMode is 'Bridge mode' or 'Layer 3 roaming'.
    type: int
  disassociateClientsOnVpnFailover:
    description: Disassociate clients when 'VPN' concentrator failover occurs in order
      to trigger clients to re-associate and generate new DHCP requests. This param
      is only valid if ipAssignmentMode is 'VPN'.
    type: bool
  dnsRewrite:
    description: DNS servers rewrite settings.
    suboptions:
      dnsCustomNameservers:
        description: User specified DNS servers (up to two servers).
        elements: str
        type: list
      enabled:
        description: Boolean indicating whether or not DNS server rewrite is enabled.
          If disabled, upstream DNS will be used.
        type: bool
    type: dict
  dot11r:
    description: The current setting for 802.11r.
    suboptions:
      adaptive:
        description: (Optional) Whether 802.11r is adaptive or not.
        type: bool
      enabled:
        description: Whether 802.11r is enabled or not.
        type: bool
    type: dict
  dot11w:
    description: The current setting for Protected Management Frames (802.11w).
    suboptions:
      enabled:
        description: Whether 802.11w is enabled or not.
        type: bool
      required:
        description: (Optional) Whether 802.11w is required or not.
        type: bool
    type: dict
  enabled:
    description: Whether or not the SSID is enabled.
    type: bool
  encryptionMode:
    description: The psk encryption mode for the SSID ('wep' or 'wpa'). This param is
      only valid if the authMode is 'psk'.
    type: str
  enterpriseAdminAccess:
    description: Whether or not an SSID is accessible by 'enterprise' administrators
      ('access disabled' or 'access enabled').
    type: str
  gre:
    description: Ethernet over GRE settings.
    suboptions:
      concentrator:
        description: The EoGRE concentrator's settings.
        suboptions:
          host:
            description: The EoGRE concentrator's IP or FQDN. This param is required
              when ipAssignmentMode is 'Ethernet over GRE'.
            type: str
        type: dict
      key:
        description: Optional numerical identifier that will add the GRE key field to
          the GRE header. Used to identify an individual traffic flow within a tunnel.
        type: int
    type: dict
  ipAssignmentMode:
    description: The client IP assignment mode ('NAT mode', 'Bridge mode', 'Layer 3
      roaming', 'Ethernet over GRE', 'Layer 3 roaming with a concentrator' or 'VPN').
    type: str
  lanIsolationEnabled:
    description: Boolean indicating whether Layer 2 LAN isolation should be enabled
      or disabled. Only configurable when ipAssignmentMode is 'Bridge mode'.
    type: bool
  ldap:
    description: The current setting for LDAP. Only valid if splashPage is 'Password-protected
      with LDAP'.
    suboptions:
      baseDistinguishedName:
        description: The base distinguished name of users on the LDAP server.
        type: str
      credentials:
        description: (Optional) The credentials of the user account to be used by the
          AP to bind to your LDAP server. The LDAP account should have permissions on
          all your LDAP servers.
        suboptions:
          distinguishedName:
            description: The distinguished name of the LDAP user account (example cn=user,dc=meraki,dc=com).
            type: str
          password:
            description: The password of the LDAP user account.
            type: str
        type: dict
      serverCaCertificate:
        description: The CA certificate used to sign the LDAP server's key.
        suboptions:
          contents:
            description: The contents of the CA certificate. Must be in PEM or DER format.
            type: str
        type: dict
      servers:
        description: The LDAP servers to be used for authentication.
        elements: dict
        suboptions:
          host:
            description: IP address of your LDAP server.
            type: str
          port:
            description: UDP port the LDAP server listens on.
            type: int
        type: list
    type: dict
  localRadius:
    description: The current setting for Local Authentication, a built-in RADIUS server
      on the access point. Only valid if authMode is '8021x-localradius'.
    suboptions:
      cacheTimeout:
        description: The duration (in seconds) for which LDAP and OCSP lookups are cached.
        type: int
      certificateAuthentication:
        description: The current setting for certificate verification.
        suboptions:
          clientRootCaCertificate:
            description: The Client CA Certificate used to sign the client certificate.
            suboptions:
              contents:
                description: The contents of the Client CA Certificate. Must be in PEM
                  or DER format.
                type: str
            type: dict
          enabled:
            description: Whether or not to use EAP-TLS certificate-based authentication
              to validate wireless clients.
            type: bool
          ocspResponderUrl:
            description: (Optional) The URL of the OCSP responder to verify client certificate
              status.
            type: str
          useLdap:
            description: Whether or not to verify the certificate with LDAP.
            type: bool
          useOcsp:
            description: Whether or not to verify the certificate with OCSP.
            type: bool
        type: dict
      passwordAuthentication:
        description: The current setting for password-based authentication.
        suboptions:
          enabled:
            description: Whether or not to use EAP-TTLS/PAP or PEAP-GTC password-based
              authentication via LDAP lookup.
            type: bool
        type: dict
    type: dict
  mandatoryDhcpEnabled:
    description: If true, Mandatory DHCP will enforce that clients connecting to this
      SSID must use the IP address assigned by the DHCP server. Clients who use a static
      IP address won't be able to associate.
    type: bool
  minBitrate:
    description: The minimum bitrate in Mbps of this SSID in the default indoor RF profile.
      ('1', '2', '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54').
    type: float
  name:
    description: The name of the SSID.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  number:
    description: Number path parameter.
    type: str
  oauth:
    description: The OAuth settings of this SSID. Only valid if splashPage is 'Google
      OAuth'.
    suboptions:
      allowedDomains:
        description: (Optional) The list of domains allowed access to the network.
        elements: str
        type: list
    type: dict
  perClientBandwidthLimitDown:
    description: The download bandwidth limit in Kbps. (0 represents no limit.).
    type: int
  perClientBandwidthLimitUp:
    description: The upload bandwidth limit in Kbps. (0 represents no limit.).
    type: int
  perSsidBandwidthLimitDown:
    description: The total download bandwidth limit in Kbps. (0 represents no limit.).
    type: int
  perSsidBandwidthLimitUp:
    description: The total upload bandwidth limit in Kbps. (0 represents no limit.).
    type: int
  psk:
    description: The passkey for the SSID. This param is only valid if the authMode
      is 'psk'.
    type: str
  radiusAccountingEnabled:
    description: Whether or not RADIUS accounting is enabled. This param is only valid
      if the authMode is 'open-with-radius', '8021x-radius' or 'ipsk-with-radius'.
    type: bool
  radiusAccountingInterimInterval:
    description: The interval (in seconds) in which accounting information is updated
      and sent to the RADIUS accounting server.
    type: int
  radiusAccountingServers:
    description: The RADIUS accounting 802.1X servers to be used for authentication.
      This param is only valid if the authMode is 'open-with-radius', '8021x-radius'
      or 'ipsk-with-radius' and radiusAccountingEnabled is 'true'.
    elements: dict
    suboptions:
      caCertificate:
        description: Certificate used for authorization for the RADSEC Server.
        type: str
      host:
        description: IP address to which the APs will send RADIUS accounting messages.
        type: str
      port:
        description: Port on the RADIUS server that is listening for accounting messages.
        type: int
      radsecEnabled:
        description: Use RADSEC (TLS over TCP) to connect to this RADIUS accounting
          server. Requires radiusProxyEnabled.
        type: bool
      secret:
        description: Shared key used to authenticate messages between the APs and RADIUS
          server.
        type: str
    type: list
  radiusAttributeForGroupPolicies:
    description: Specify the RADIUS attribute used to look up group policies ('Filter-Id',
      'Reply-Message', 'Airespace-ACL-Name' or 'Aruba-User-Role'). Access points must
      receive this attribute in the RADIUS Access-Accept message.
    type: str
  radiusAuthenticationNasId:
    description: The template of the NAS identifier to be used for RADIUS authentication
      (ex. $NODE_MAC$ $VAP_NUM$).
    type: str
  radiusCalledStationId:
    description: The template of the called station identifier to be used for RADIUS
      (ex. $NODE_MAC$ $VAP_NUM$).
    type: str
  radiusCoaEnabled:
    description: If true, Meraki devices will act as a RADIUS Dynamic Authorization
      Server and will respond to RADIUS Change-of-Authorization and Disconnect messages
      sent by the RADIUS server.
    type: bool
  radiusFailoverPolicy:
    description: This policy determines how authentication requests should be handled
      in the event that all of the configured RADIUS servers are unreachable ('Deny
      access' or 'Allow access').
    type: str
  radiusFallbackEnabled:
    description: Whether or not higher priority RADIUS servers should be retried after
      60 seconds.
    type: bool
  radiusGuestVlanEnabled:
    description: Whether or not RADIUS Guest VLAN is enabled. This param is only valid
      if the authMode is 'open-with-radius' and addressing mode is not set to 'isolated'
      or 'nat' mode.
    type: bool
  radiusGuestVlanId:
    description: VLAN ID of the RADIUS Guest VLAN. This param is only valid if the authMode
      is 'open-with-radius' and addressing mode is not set to 'isolated' or 'nat' mode.
    type: int
  radiusLoadBalancingPolicy:
    description: This policy determines which RADIUS server will be contacted first
      in an authentication attempt and the ordering of any necessary retry attempts
      ('Strict priority order' or 'Round robin').
    type: str
  radiusOverride:
    description: If true, the RADIUS response can override VLAN tag. This is not valid
      when ipAssignmentMode is 'NAT mode'.
    type: bool
  radiusProxyEnabled:
    description: If true, Meraki devices will proxy RADIUS messages through the Meraki
      cloud to the configured RADIUS auth and accounting servers.
    type: bool
  radiusServerAttemptsLimit:
    description: The maximum number of transmit attempts after which a RADIUS server
      is failed over (must be between 1-5).
    type: int
  radiusServerTimeout:
    description: The amount of time for which a RADIUS client waits for a reply from
      the RADIUS server (must be between 1-10 seconds).
    type: int
  radiusServers:
    description: The RADIUS 802.1X servers to be used for authentication. This param
      is only valid if the authMode is 'open-with-radius', '8021x-radius' or 'ipsk-with-radius'.
    elements: dict
    suboptions:
      caCertificate:
        description: Certificate used for authorization for the RADSEC Server.
        type: str
      host:
        description: IP address of your RADIUS server.
        type: str
      openRoamingCertificateId:
        description: The ID of the Openroaming Certificate attached to radius server.
        type: int
      port:
        description: UDP port the RADIUS server listens on for Access-requests.
        type: int
      radsecEnabled:
        description: Use RADSEC (TLS over TCP) to connect to this RADIUS server. Requires
          radiusProxyEnabled.
        type: bool
      secret:
        description: RADIUS client shared secret.
        type: str
    type: list
  radiusTestingEnabled:
    description: If true, Meraki devices will periodically send Access-Request messages
      to configured RADIUS servers using identity 'meraki_8021x_test' to ensure that
      the RADIUS servers are reachable.
    type: bool
  secondaryConcentratorNetworkId:
    description: The secondary concentrator to use when the ipAssignmentMode is 'VPN'.
      If configured, the APs will switch to using this concentrator if the primary concentrator
      is unreachable. This param is optional. ('disabled' represents no secondary concentrator.).
    type: str
  speedBurst:
    description: The SpeedBurst setting for this SSID'.
    suboptions:
      enabled:
        description: Boolean indicating whether or not to allow users to temporarily
          exceed the bandwidth limit for short periods while still keeping them under
          the bandwidth limit over time.
        type: bool
    type: dict
  splashGuestSponsorDomains:
    description: Array of valid sponsor email domains for sponsored guest splash type.
    elements: str
    type: list
  splashPage:
    description: The type of splash page for the SSID ('None', 'Click-through splash
      page', 'Billing', 'Password-protected with Meraki RADIUS', 'Password-protected
      with custom RADIUS', 'Password-protected with Active Directory', 'Password-protected
      with LDAP', 'SMS authentication', 'Systems Manager Sentry', 'Facebook Wi-Fi',
      'Google OAuth', 'Sponsored guest', 'Cisco ISE' or 'Google Apps domain'). This
      attribute is not supported for template children.
    type: str
  useVlanTagging:
    description: Whether or not traffic should be directed to use specific VLANs. This
      param is only valid if the ipAssignmentMode is 'Bridge mode' or 'Layer 3 roaming'.
    type: bool
  visible:
    description: Boolean indicating whether APs should advertise or hide this SSID.
      APs will only broadcast this SSID if set to true.
    type: bool
  vlanId:
    description: The VLAN ID used for VLAN tagging. This param is only valid when the
      ipAssignmentMode is 'Layer 3 roaming with a concentrator' or 'VPN'.
    type: int
  walledGardenEnabled:
    description: Allow access to a configurable list of IP ranges, which users may access
      prior to sign-on.
    type: bool
  walledGardenRanges:
    description: Specify your walled garden by entering an array of addresses, ranges
      using CIDR notation, domain names, and domain wildcards (e.g. '192.168.1.1/24',
      '192.168.37.10/32', 'www.yahoo.com', '*.google.com'). Meraki's splash page is
      automatically included in your walled garden.
    elements: str
    type: list
  wpaEncryptionMode:
    description: The types of WPA encryption. ('WPA1 only', 'WPA1 and WPA2', 'WPA2 only',
      'WPA3 Transition Mode', 'WPA3 only' or 'WPA3 192-bit Security').
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless updateNetworkWirelessSsid
  description: Complete reference of the updateNetworkWirelessSsid API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-ssid
notes:
  - SDK Method used are
    wireless.Wireless.update_network_wireless_ssid,

  - Paths used are
    put /networks/{networkId}/wireless/ssids/{number},
"""

EXAMPLES = r"""
- name: Update by id
  cisco.meraki.networks_wireless_ssids:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    enabled: true
    name: My SSID
    networkId: string
    number: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wireless_accesspoint_configuration_summary_info
short_description: Information module for Wireless Accesspoint Configuration Summary
description:
- Get all Wireless Accesspoint Configuration Summary.
- Users can query the access point configuration information per device using the ethernet MAC address.
version_added: '6.7.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  key:
    description:
    - Key query parameter. The ethernet MAC address of Access point.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Wireless GetAccessPointConfiguration
  description: Complete reference of the GetAccessPointConfiguration API.
  link: https://developer.cisco.com/docs/dna-center/#!get-access-point-configuration
notes:
  - SDK Method used are
    wireless.Wireless.get_access_point_configuration,

  - Paths used are
    get /dna/intent/api/v1/wireless/accesspoint-configuration/summary,

"""

EXAMPLES = r"""
- name: Get all Wireless Accesspoint Configuration Summary
  cisco.dnac.wireless_accesspoint_configuration_summary_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    key: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "instanceUuid": {},
      "instanceId": 0,
      "authEntityId": {},
      "displayName": "string",
      "authEntityClass": {},
      "instanceTenantId": "string",
      "_orderedListOEIndex": 0,
      "_orderedListOEAssocName": {},
      "_creationOrderIndex": 0,
      "_isBeingChanged": true,
      "deployPending": "string",
      "instanceCreatedOn": {},
      "instanceUpdatedOn": {},
      "changeLogList": {},
      "instanceOrigin": {},
      "lazyLoadedEntities": {},
      "instanceVersion": 0,
      "adminStatus": "string",
      "apHeight": 0,
      "apMode": "string",
      "apName": "string",
      "ethMac": "string",
      "failoverPriority": "string",
      "ledBrightnessLevel": 0,
      "ledStatus": "string",
      "location": "string",
      "macAddress": "string",
      "primaryControllerName": "string",
      "primaryIpAddress": "string",
      "secondaryControllerName": "string",
      "secondaryIpAddress": "string",
      "tertiaryControllerName": "string",
      "tertiaryIpAddress": "string",
      "meshDTOs": [
        {}
      ],
      "radioDTOs": [
        {
          "instanceUuid": {},
          "instanceId": 0,
          "authEntityId": {},
          "displayName": "string",
          "authEntityClass": {},
          "instanceTenantId": "string",
          "_orderedListOEIndex": 0,
          "_orderedListOEAssocName": {},
          "_creationOrderIndex": 0,
          "_isBeingChanged": true,
          "deployPending": "string",
          "instanceCreatedOn": {},
          "instanceUpdatedOn": {},
          "changeLogList": {},
          "instanceOrigin": {},
          "lazyLoadedEntities": {},
          "instanceVersion": 0,
          "adminStatus": "string",
          "antennaAngle": 0,
          "antennaElevAngle": 0,
          "antennaGain": 0,
          "antennaPatternName": "string",
          "channelAssignmentMode": "string",
          "channelNumber": 0,
          "channelWidth": "string",
          "cleanAirSI": "string",
          "ifType": 0,
          "ifTypeValue": "string",
          "macAddress": "string",
          "powerAssignmentMode": "string",
          "powerlevel": 0,
          "radioBand": {},
          "radioRoleAssignment": {},
          "slotId": 0,
          "internalKey": {
            "type": "string",
            "id": 0,
            "longType": "string",
            "url": "string"
          }
        }
      ],
      "internalKey": {
        "type": "string",
        "id": 0,
        "longType": "string",
        "url": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: compliance_device_info
short_description: Information module for Compliance Device
description:
- Get all Compliance Device.
- Get Compliance Device by id.
- Return compliance status of a device.
- Return compliance status of devices.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  complianceStatus:
    description:
    - >
      ComplianceStatus query parameter. Compliance status can be have value among
      'COMPLIANT','NON_COMPLIANT','IN_PROGRESS', 'ERROR'.
    type: str
  deviceUuid:
    description:
    - DeviceUuid query parameter. Comma separated deviceUuids.
    type: str
  offset:
    description:
    - Offset query parameter. Offset/starting row.
    type: int
  limit:
    description:
    - Limit query parameter. Number of records to be retrieved.
    type: int
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Compliance DeviceComplianceStatus
  description: Complete reference of the DeviceComplianceStatus API.
  link: https://developer.cisco.com/docs/dna-center/#!device-compliance-status
- name: Cisco DNA Center documentation for Compliance GetComplianceStatus
  description: Complete reference of the GetComplianceStatus API.
  link: https://developer.cisco.com/docs/dna-center/#!get-compliance-status
notes:
  - SDK Method used are
    compliance.Compliance.device_compliance_status,
    compliance.Compliance.get_compliance_status,

  - Paths used are
    get /dna/intent/api/v1/compliance,
    get /dna/intent/api/v1/compliance/{deviceUuid},

"""

EXAMPLES = r"""
- name: Get all Compliance Device
  cisco.dnac.compliance_device_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    complianceStatus: string
    deviceUuid: string
    offset: 0
    limit: 0
  register: result

- name: Get Compliance Device by id
  cisco.dnac.compliance_device_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceUuid: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "deviceUuid": "string",
        "complianceStatus": "string",
        "message": "string",
        "scheduleTime": 0,
        "lastUpdateTime": 0
      }
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: bulk_job_launch
author: "Seth Foster (@fosterseth)"
short_description: Bulk job launch in Automation Platform Controller
description:
    - Single-request bulk job launch in Automation Platform Controller.
    - Creates a workflow where each node corresponds to an item specified in the jobs option.
    - Any options specified at the top level will inherited by the launched jobs (if prompt on launch is enabled for those fields).
    - Provides a way to submit many jobs at once to Controller.
options:
    jobs:
      description:
        - List of jobs to create.
      required: True
      type: list
      elements: dict
      suboptions:
        unified_job_template:
          description:
            - Job template ID to use when launching.
          type: int
          required: True
        inventory:
          description:
            - Inventory ID applied as a prompt, if job template prompts for inventory
          type: int
        execution_environment:
          description:
            - Execution environment ID applied as a prompt, if job template prompts for execution environments
          type: int
        instance_groups:
          description:
            - Instance group IDs applied as a prompt, if job template prompts for instance groups
          type: list
          elements: int
        credentials:
          description:
            - Credential IDs applied as a prompt, if job template prompts for credentials
          type: list
          elements: int
        labels:
          description:
            - Label IDs to use for the job, if job template prompts for labels
          type: list
          elements: int
        extra_data:
          description:
            - Extra variables to apply at launch time, if job template prompts for extra variables
          type: dict
          default: {}
        diff_mode:
          description:
            - Show the changes made by Ansible tasks where supported
          type: bool
        verbosity:
          description:
            - Verbosity level for this ad hoc command run
          type: int
          choices: [ 0, 1, 2, 3, 4, 5 ]
        scm_branch:
          description:
            - SCM branch applied as a prompt, if job template prompts for SCM branch
            - This is only applicable if the project allows for branch override
          type: str
        job_type:
          description:
            - Job type applied as a prompt, if job template prompts for job type
          type: str
          choices:
            - 'run'
            - 'check'
        job_tags:
          description:
            - Job tags applied as a prompt, if job template prompts for job tags
          type: str
        skip_tags:
          description:
            - Tags to skip, applied as a prompt, if job template prompts for job tags
          type: str
        limit:
          description:
            - Limit to act on, applied as a prompt, if job template prompts for limit
          type: str
        forks:
          description:
            - The number of parallel or simultaneous processes to use while executing the playbook, if job template prompts for forks
          type: int
        job_slice_count:
          description:
            - The number of jobs to slice into at runtime, if job template prompts for job slices.
            - Will cause the Job Template to launch a workflow if value is greater than 1.
          type: int
          default: '1'
        identifier:
          description:
            - Identifier for the resulting workflow node that represents this job
          type: str
        timeout:
          description:
            - Maximum time in seconds to wait for a job to finish (server-side), if job template prompts for timeout.
          type: int
    name:
      description:
        - The name of the bulk job that is created
      required: False
      type: str
    description:
      description:
        - Optional description of this bulk job.
      type: str
    organization:
      description:
        - If not provided, will use the organization the user is in.
        - Required if the user belongs to more than one organization.
        - Affects who can see the resulting bulk job.
      type: str
    inventory:
      description:
        - Inventory name, ID, or named URL to use for the jobs ran within the bulk job, only used if prompt for inventory is set.
      type: str
    scm_branch:
      description:
        - A specific branch of the SCM project to run the template on.
        - This is only applicable if the project allows for branch override.
      type: str
    extra_vars:
      description:
        - Any extra vars required to launch the job.
        - Extends the extra_data field at the individual job level.
      type: dict
    limit:
      description:
        - Limit to use for the bulk job.
      type: str
    job_tags:
      description:
        - A comma-separated list of playbook tags to specify what parts of the playbooks should be executed.
      type: str
    skip_tags:
      description:
        - A comma-separated list of playbook tags to skip certain tasks or parts of the playbooks to be executed.
      type: str
    wait:
      description:
        - Wait for the bulk job to complete.
      default: True
      type: bool
    interval:
      description:
        - The interval to request an update from the controller.
      required: False
      default: 2
      type: float
extends_documentation_fragment: awx.awx.auth
'''

RETURN = '''
job_info:
    description: dictionary containing information about the bulk job executed
    returned: If bulk job launched
    type: dict
'''

EXAMPLES = '''
- name: Launch bulk jobs
  bulk_job_launch:
    name: My Bulk Job Launch
    jobs:
      - unified_job_template: 7
        skip_tags: foo
      - unified_job_template: 10
        limit: foo
        extra_data:
          food: carrot
          color: orange
    limit: bar
    credentials:
      - "My Credential"
      - "suplementary cred"
    extra_vars: # these override / extend extra_data at the job level
      food: grape
      animal: owl
    organization: Default
    inventory: Demo Inventory

- name: Launch bulk jobs with lookup plugin
  bulk_job_launch:
    name: My Bulk Job Launch
    jobs:
      - unified_job_template: 7
      - unified_job_template: "{{ lookup('awx.awx.controller_api', 'job_templates', query_params={'name': 'Demo Job Template'},
        return_ids=True, expect_one=True) }}"
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        jobs=dict(required=True, type='list', elements='dict'),
        name=dict(),
        description=dict(),
        organization=dict(type='str'),
        inventory=dict(type='str'),
        limit=dict(),
        scm_branch=dict(),
        extra_vars=dict(type='dict'),
        job_tags=dict(),
        skip_tags=dict(),
        wait=dict(required=False, default=True, type='bool'),
        interval=dict(required=False, default=2.0, type='float'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    post_data_names = (
        'jobs',
        'name',
        'description',
        'limit',
        'scm_branch',
        'extra_vars',
        'job_tags',
        'skip_tags',
    )
    post_data = {}
    for p in post_data_names:
        val = module.params.get(p)
        if val:
            post_data[p] = val

    # Resolve name to ID for related resources
    # Do not resolve name for "jobs" suboptions, for optimization
    org_name = module.params.get('organization')
    if org_name:
        post_data['organization'] = module.resolve_name_to_id('organizations', org_name)

    inv_name = module.params.get('inventory')
    if inv_name:
        post_data['inventory'] = module.resolve_name_to_id('inventories', inv_name)

    # Extract our parameters
    wait = module.params.get('wait')
    timeout = module.params.get('timeout')
    interval = module.params.get('interval')
    name = module.params.get('name')

    # Launch the jobs
    result = module.post_endpoint("bulk/job_launch", data=post_data)

    if result['status_code'] != 201:
        module.fail_json(msg="Failed to launch bulk jobs, see response for details", response=result)

    module.json_output['changed'] = True
    module.json_output['id'] = result['json']['id']
    module.json_output['status'] = result['json']['status']
    # This is for backwards compatability
    module.json_output['job_info'] = result['json']

    if not wait:
        module.exit_json(**module.json_output)

    # Invoke wait function
    module.wait_on_url(url=result['json']['url'], object_name=name, object_type='Bulk Job Launch', timeout=timeout, interval=interval)

    module.exit_json(**module.json_output)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment

    # NOTE: This document fragment needs to be augmented by ZONE_ID_TYPE in a provider document fragment.
    #       The ZONE_ID_TYPE fragment will provide `choices` for the options.type entry.
    DOCUMENTATION = r'''
options:
    what:
        description:
          - Describes whether to fetch a single record and type combination, all types for a
            record, or all records. By default, a single record and type combination is fetched.
          - Note that the return value structure depends on this option.
        choices: ['single_record', 'all_types_for_record', 'all_records']
        default: single_record
        type: str
    zone_name:
        description:
          - The DNS zone to modify.
          - Exactly one of O(zone_name) and O(zone_id) must be specified.
        type: str
        aliases:
          - zone
    zone_id:
        description:
          - The ID of the DNS zone to modify.
          - Exactly one of O(zone_name) and O(zone_id) must be specified.
        version_added: 0.2.0
    record:
        description:
          - The full DNS record to retrieve.
          - If O(what) is V(single_record) or V(all_types_for_record), exactly one of O(record) and O(prefix) is required.
        type: str
    prefix:
        description:
          - The prefix of the DNS record.
          - This is the part of O(record) before O(zone_name). For example, if the record to be modified is C(www.example.com)
            for the zone C(example.com), the prefix is V(www). If the record in this example would be C(example.com), the
            prefix would be V('') (empty string).
          - If O(what) is V(single_record) or V(all_types_for_record), exactly one of O(record) and O(prefix) is required.
        type: str
        version_added: 0.2.0
    type:
        description:
          - The type of DNS record to retrieve.
          - Required if O(what) is V(single_record).
        type: str
'''



# -*- coding: utf-8 -*-
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

DOCUMENTATION = r'''
    name: scaleway
    author:
      - Remy Leone (@remyleone)
    short_description: Scaleway inventory source
    description:
        - Get inventory hosts from Scaleway.
    requirements:
        - PyYAML
    options:
        plugin:
            description: Token that ensures this is a source file for the 'scaleway' plugin.
            required: true
            choices: ['scaleway', 'community.general.scaleway']
        regions:
            description: Filter results on a specific Scaleway region.
            type: list
            elements: string
            default:
                - ams1
                - par1
                - par2
                - waw1
        tags:
            description: Filter results on a specific tag.
            type: list
            elements: string
        scw_profile:
            description:
            - The config profile to use in config file.
            - By default uses the one specified as C(active_profile) in the config file, or falls back to V(default) if that is not defined.
            type: string
            version_added: 4.4.0
        oauth_token:
            description:
            - Scaleway OAuth token.
            - If not explicitly defined or in environment variables, it will try to lookup in the scaleway-cli configuration file
              (C($SCW_CONFIG_PATH), C($XDG_CONFIG_HOME/scw/config.yaml), or C(~/.config/scw/config.yaml)).
            - More details on L(how to generate token, https://www.scaleway.com/en/docs/generate-api-keys/).
            env:
                # in order of precedence
                - name: SCW_TOKEN
                - name: SCW_API_KEY
                - name: SCW_OAUTH_TOKEN
        hostnames:
            description: List of preference about what to use as an hostname.
            type: list
            elements: string
            default:
                - public_ipv4
            choices:
                - public_ipv4
                - private_ipv4
                - public_ipv6
                - hostname
                - id
        variables:
            description: 'Set individual variables: keys are variable names and
                          values are templates. Any value returned by the
                          L(Scaleway API, https://developer.scaleway.com/#servers-server-get)
                          can be used.'
            type: dict
'''

EXAMPLES = r'''
# scaleway_inventory.yml file in YAML format
# Example command line: ansible-inventory --list -i scaleway_inventory.yml

# use hostname as inventory_hostname
# use the private IP address to connect to the host
plugin: community.general.scaleway
regions:
  - ams1
  - par1
tags:
  - foobar
hostnames:
  - hostname
variables:
  ansible_host: private_ip
  state: state

# use hostname as inventory_hostname and public IP address to connect to the host
plugin: community.general.scaleway
hostnames:
  - hostname
regions:
  - par1
variables:
  ansible_host: public_ip.address

# Using static strings as variables
plugin: community.general.scaleway
hostnames:
  - hostname
variables:
  ansible_host: public_ip.address
  ansible_connection: "'ssh'"
  ansible_user: "'admin'"
'''

import os
import json

try:
    import yaml
except ImportError as exc:
    YAML_IMPORT_ERROR = exc
else:
    YAML_IMPORT_ERROR = None

from ansible.errors import AnsibleError
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable
from ansible_collections.community.general.plugins.module_utils.scaleway import SCALEWAY_LOCATION, parse_pagination_link
from ansible.module_utils.urls import open_url
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.module_utils.six import raise_from
from ansible.utils.unsafe_proxy import wrap_var as make_unsafe

import ansible.module_utils.six.moves.urllib.parse as urllib_parse


def _fetch_information(token, url):
    results = []
    paginated_url = url
    while True:
        try:
            response = open_url(paginated_url,
                                headers={'X-Auth-Token': token,
                                         'Content-type': 'application/json'})
        except Exception as e:
            raise AnsibleError("Error while fetching %s: %s" % (url, to_native(e)))
        try:
            raw_json = json.loads(to_text(response.read()))
        except ValueError:
            raise AnsibleError("Incorrect JSON payload")

        try:
            results.extend(raw_json["servers"])
        except KeyError:
            raise AnsibleError("Incorrect format from the Scaleway API response")

        link = response.headers['Link']
        if not link:
            return results
        relations = parse_pagination_link(link)
        if 'next' not in relations:
            return results
        paginated_url = urllib_parse.urljoin(paginated_url, relations['next'])


def _build_server_url(api_endpoint):
    return "/".join([api_endpoint, "servers"])


def extract_public_ipv4(server_info):
    try:
        return server_info["public_ip"]["address"]
    except (KeyError, TypeError):
        return None


def extract_private_ipv4(server_info):
    try:
        return server_info["private_ip"]
    except (KeyError, TypeError):
        return None


def extract_hostname(server_info):
    try:
        return server_info["hostname"]
    except (KeyError, TypeError):
        return None


def extract_server_id(server_info):
    try:
        return server_info["id"]
    except (KeyError, TypeError):
        return None


def extract_public_ipv6(server_info):
    try:
        return server_info["ipv6"]["address"]
    except (KeyError, TypeError):
        return None


def extract_tags(server_info):
    try:
        return server_info["tags"]
    except (KeyError, TypeError):
        return None


def extract_zone(server_info):
    try:
        return server_info["location"]["zone_id"]
    except (KeyError, TypeError):
        return None


extractors = {
    "public_ipv4": extract_public_ipv4,
    "private_ipv4": extract_private_ipv4,
    "public_ipv6": extract_public_ipv6,
    "hostname": extract_hostname,
    "id": extract_server_id
}


class InventoryModule(BaseInventoryPlugin, Constructable):
    NAME = 'community.general.scaleway'

    def _fill_host_variables(self, host, server_info):
        targeted_attributes = (
            "arch",
            "commercial_type",
            "id",
            "organization",
            "state",
            "hostname",
        )
        for attribute in targeted_attributes:
            self.inventory.set_variable(host, attribute, server_info[attribute])

        self.inventory.set_variable(host, "tags", server_info["tags"])

        if extract_public_ipv6(server_info=server_info):
            self.inventory.set_variable(host, "public_ipv6", extract_public_ipv6(server_info=server_info))

        if extract_public_ipv4(server_info=server_info):
            self.inventory.set_variable(host, "public_ipv4", extract_public_ipv4(server_info=server_info))

        if extract_private_ipv4(server_info=server_info):
            self.inventory.set_variable(host, "private_ipv4", extract_private_ipv4(server_info=server_info))

    def _get_zones(self, config_zones):
        return set(SCALEWAY_LOCATION.keys()).intersection(config_zones)

    def match_groups(self, server_info, tags):
        server_zone = extract_zone(server_info=server_info)
        server_tags = extract_tags(server_info=server_info)

        # If a server does not have a zone, it means it is archived
        if server_zone is None:
            return set()

        # If no filtering is defined, all tags are valid groups
        if tags is None:
            return set(server_tags).union((server_zone,))

        matching_tags = set(server_tags).intersection(tags)

        if not matching_tags:
            return set()
        return matching_tags.union((server_zone,))

    def _filter_host(self, host_infos, hostname_preferences):

        for pref in hostname_preferences:
            if extractors[pref](host_infos):
                return extractors[pref](host_infos)

        return None

    def do_zone_inventory(self, zone, token, tags, hostname_preferences):
        self.inventory.add_group(zone)
        zone_info = SCALEWAY_LOCATION[zone]

        url = _build_server_url(zone_info["api_endpoint"])
        raw_zone_hosts_infos = make_unsafe(_fetch_information(url=url, token=token))

        for host_infos in raw_zone_hosts_infos:

            hostname = self._filter_host(host_infos=host_infos,
                                         hostname_preferences=hostname_preferences)

            # No suitable hostname were found in the attributes and the host won't be in the inventory
            if not hostname:
                continue

            groups = self.match_groups(host_infos, tags)

            for group in groups:
                self.inventory.add_group(group=group)
                self.inventory.add_host(group=group, host=hostname)
                self._fill_host_variables(host=hostname, server_info=host_infos)

                # Composed variables
                self._set_composite_vars(self.get_option('variables'), host_infos, hostname, strict=False)

    def get_oauth_token(self):
        oauth_token = self.get_option('oauth_token')

        if 'SCW_CONFIG_PATH' in os.environ:
            scw_config_path = os.getenv('SCW_CONFIG_PATH')
        elif 'XDG_CONFIG_HOME' in os.environ:
            scw_config_path = os.path.join(os.getenv('XDG_CONFIG_HOME'), 'scw', 'config.yaml')
        else:
            scw_config_path = os.path.join(os.path.expanduser('~'), '.config', 'scw', 'config.yaml')

        if not oauth_token and os.path.exists(scw_config_path):
            with open(scw_config_path) as fh:
                scw_config = yaml.safe_load(fh)
                ansible_profile = self.get_option('scw_profile')

                if ansible_profile:
                    active_profile = ansible_profile
                else:
                    active_profile = scw_config.get('active_profile', 'default')

                if active_profile == 'default':
                    oauth_token = scw_config.get('secret_key')
                else:
                    oauth_token = scw_config['profiles'][active_profile].get('secret_key')

        return oauth_token

    def parse(self, inventory, loader, path, cache=True):
        if YAML_IMPORT_ERROR:
            raise_from(AnsibleError('PyYAML is probably missing'), YAML_IMPORT_ERROR)
        super(InventoryModule, self).parse(inventory, loader, path)
        self._read_config_data(path=path)

        config_zones = self.get_option("regions")
        tags = self.get_option("tags")
        token = self.get_oauth_token()
        if not token:
            raise AnsibleError("'oauth_token' value is null, you must configure it either in inventory, envvars or scaleway-cli config.")
        hostname_preference = self.get_option("hostnames")

        for zone in self._get_zones(config_zones):
            self.do_zone_inventory(zone=make_unsafe(zone), token=token, tags=tags, hostname_preferences=hostname_preference)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_vpc_igw
version_added: 1.0.0
short_description: Manage an AWS VPC Internet gateway
description:
    - Manage an AWS VPC Internet gateway
author: Robert Estelle (@erydo)
options:
  internet_gateway_id:
    version_added: 7.0.0
    description:
      - The ID of Internet Gateway to manage.
    required: false
    type: str
  vpc_id:
    description:
      - The VPC ID for the VPC to attach (when state=present)
      - VPC ID can also be provided to find the internet gateway to manage that the VPC is attached to
    required: false
    type: str
  state:
    description:
      - Create or terminate the IGW
    default: present
    choices: [ 'present', 'absent' ]
    type: str
  force_attach:
    version_added: 7.0.0
    description:
      - Force attaching VPC to I(vpc_id).
      - Setting this option to true will detach an existing VPC attachment and attach to the supplied I(vpc_id).
      - Ignored when I(state=absent).
      - I(vpc_id) must be specified when I(force_attach) is true
    default: false
    type: bool
  detach_vpc:
    version_added: 7.0.0
    description:
      - Remove attached VPC from gateway
    default: false
    type: bool
notes:
- Support for I(purge_tags) was added in release 1.3.0.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# Ensure that the VPC has an Internet Gateway.
# The Internet Gateway ID is can be accessed via {{igw.gateway_id}} for use in setting up NATs etc.
- name: Create Internet gateway
  amazon.aws.ec2_vpc_igw:
    vpc_id: vpc-abcdefgh
    state: present
  register: igw

- name: Create Internet gateway with tags
  amazon.aws.ec2_vpc_igw:
    vpc_id: vpc-abcdefgh
    state: present
    tags:
      Tag1: tag1
      Tag2: tag2
  register: igw

- name: Create a detached gateway
  amazon.aws.ec2_vpc_igw:
    state: present
  register: igw

- name: Change the VPC the gateway is attached to
  amazon.aws.ec2_vpc_igw:
    internet_gateway_id: igw-abcdefgh
    vpc_id: vpc-stuvwxyz
    force_attach: true
    state: present
  register: igw

- name: Delete Internet gateway using the attached vpc id
  amazon.aws.ec2_vpc_igw:
    state: absent
    vpc_id: vpc-abcdefgh
  register: vpc_igw_delete

- name: Delete Internet gateway with gateway id
  amazon.aws.ec2_vpc_igw:
    state: absent
    internet_gateway_id: igw-abcdefgh
  register: vpc_igw_delete

- name: Delete Internet gateway ensuring attached VPC is correct
  amazon.aws.ec2_vpc_igw:
    state: absent
    internet_gateway_id: igw-abcdefgh
    vpc_id: vpc-abcdefgh
  register: vpc_igw_delete
"""

RETURN = r"""
changed:
  description: If any changes have been made to the Internet Gateway.
  type: bool
  returned: always
  sample:
    changed: false
gateway_id:
  description: The unique identifier for the Internet Gateway.
  type: str
  returned: I(state=present)
  sample:
    gateway_id: "igw-XXXXXXXX"
tags:
  description: The tags associated the Internet Gateway.
  type: dict
  returned: I(state=present)
  sample:
    tags:
      "Ansible": "Test"
vpc_id:
  description: The VPC ID associated with the Internet Gateway.
  type: str
  returned: I(state=present)
  sample:
    vpc_id: "vpc-XXXXXXXX"
"""

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter


@AWSRetry.jittered_backoff(retries=10, delay=10)
def describe_igws_with_backoff(connection, **params):
    paginator = connection.get_paginator("describe_internet_gateways")
    return paginator.paginate(**params).build_full_result()["InternetGateways"]


def describe_vpcs_with_backoff(connection, **params):
    paginator = connection.get_paginator("describe_vpcs")
    return paginator.paginate(**params).build_full_result()["Vpcs"]


class AnsibleEc2Igw:
    def __init__(self, module, results):
        self._module = module
        self._results = results
        self._connection = self._module.client("ec2", retry_decorator=AWSRetry.jittered_backoff())
        self._check_mode = self._module.check_mode

    def process(self):
        internet_gateway_id = self._module.params.get("internet_gateway_id")
        vpc_id = self._module.params.get("vpc_id")
        state = self._module.params.get("state", "present")
        tags = self._module.params.get("tags")
        purge_tags = self._module.params.get("purge_tags")
        force_attach = self._module.params.get("force_attach")
        detach_vpc = self._module.params.get("detach_vpc")

        if state == "present":
            self.ensure_igw_present(internet_gateway_id, vpc_id, tags, purge_tags, force_attach, detach_vpc)
        elif state == "absent":
            self.ensure_igw_absent(internet_gateway_id, vpc_id)

    def get_matching_igw(self, vpc_id, gateway_id=None):
        """
        Returns the internet gateway found.
            Parameters:
                vpc_id (str): VPC ID
                gateway_id (str): Internet Gateway ID, if specified
            Returns:
                igw (dict): dict of igw found, None if none found
        """
        try:
            # If we know the gateway_id, use it to avoid bugs with using filters
            # See https://github.com/ansible-collections/amazon.aws/pull/766
            if not gateway_id:
                filters = ansible_dict_to_boto3_filter_list({"attachment.vpc-id": vpc_id})
                igws = describe_igws_with_backoff(self._connection, Filters=filters)
            else:
                igws = describe_igws_with_backoff(self._connection, InternetGatewayIds=[gateway_id])
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self._module.fail_json_aws(e)

        igw = None
        if len(igws) > 1:
            self._module.fail_json(msg=f"EC2 returned more than one Internet Gateway for VPC {vpc_id}, aborting")
        elif igws:
            igw = camel_dict_to_snake_dict(igws[0])

        return igw

    def get_matching_vpc(self, vpc_id):
        """
        Returns the virtual private cloud found.
            Parameters:
                vpc_id (str): VPC ID
            Returns:
                vpc (dict): dict of vpc found, None if none found
        """
        try:
            vpcs = describe_vpcs_with_backoff(self._connection, VpcIds=[vpc_id])
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            # self._module.fail_json(msg=f"{str(e)}")
            if "InvalidVpcID.NotFound" in str(e):
                self._module.fail_json(msg=f"VPC with Id {vpc_id} not found, aborting")
            self._module.fail_json_aws(e)

        vpc = None
        if len(vpcs) > 1:
            self._module.fail_json(msg=f"EC2 returned more than one VPC for {vpc_id}, aborting")
        elif vpcs:
            vpc = camel_dict_to_snake_dict(vpcs[0])

        return vpc

    @staticmethod
    def get_igw_info(igw, vpc_id):
        return {
            "gateway_id": igw["internet_gateway_id"],
            "tags": boto3_tag_list_to_ansible_dict(igw["tags"]),
            "vpc_id": vpc_id,
        }

    def detach_vpc(self, igw_id, vpc_id):
        try:
            self._connection.detach_internet_gateway(aws_retry=True, InternetGatewayId=igw_id, VpcId=vpc_id)

            self._results["changed"] = True
        except botocore.exceptions.WaiterError as e:
            self._module.fail_json_aws(e, msg="Unable to detach VPC.")

    def attach_vpc(self, igw_id, vpc_id):
        try:
            self._connection.attach_internet_gateway(aws_retry=True, InternetGatewayId=igw_id, VpcId=vpc_id)

            # Ensure the gateway is attached before proceeding
            waiter = get_waiter(self._connection, "internet_gateway_attached")
            waiter.wait(InternetGatewayIds=[igw_id])

            self._results["changed"] = True
        except botocore.exceptions.WaiterError as e:
            self._module.fail_json_aws(e, msg="Failed to attach VPC.")

    def ensure_igw_absent(self, igw_id, vpc_id):
        igw = self.get_matching_igw(vpc_id, gateway_id=igw_id)
        if igw is None:
            return self._results

        igw_vpc_id = ""

        if len(igw["attachments"]) > 0:
            igw_vpc_id = igw["attachments"][0]["vpc_id"]

        if vpc_id and (igw_vpc_id != vpc_id):
            self._module.fail_json(msg=f"Supplied VPC ({vpc_id}) does not match found VPC ({igw_vpc_id}), aborting")

        if self._check_mode:
            self._results["changed"] = True
            return self._results

        try:
            self._results["changed"] = True

            if igw_vpc_id:
                self.detach_vpc(igw["internet_gateway_id"], igw_vpc_id)

            self._connection.delete_internet_gateway(aws_retry=True, InternetGatewayId=igw["internet_gateway_id"])
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            self._module.fail_json_aws(e, msg="Unable to delete Internet Gateway")

        return self._results

    def ensure_igw_present(self, igw_id, vpc_id, tags, purge_tags, force_attach, detach_vpc):
        igw = None

        if igw_id:
            igw = self.get_matching_igw(None, gateway_id=igw_id)
        elif vpc_id:
            igw = self.get_matching_igw(vpc_id)

        if igw is None:
            if self._check_mode:
                self._results["changed"] = True
                self._results["gateway_id"] = None
                return self._results

            if vpc_id:
                self.get_matching_vpc(vpc_id)

            try:
                create_params = {}
                if tags:
                    create_params["TagSpecifications"] = boto3_tag_specifications(tags, types="internet-gateway")
                response = self._connection.create_internet_gateway(aws_retry=True, **create_params)

                # Ensure the gateway exists before trying to attach it or add tags
                waiter = get_waiter(self._connection, "internet_gateway_exists")
                waiter.wait(InternetGatewayIds=[response["InternetGateway"]["InternetGatewayId"]])
                self._results["changed"] = True

                igw = camel_dict_to_snake_dict(response["InternetGateway"])

                if vpc_id:
                    self.attach_vpc(igw["internet_gateway_id"], vpc_id)
            except botocore.exceptions.WaiterError as e:
                self._module.fail_json_aws(e, msg="No Internet Gateway exists.")
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                self._module.fail_json_aws(e, msg="Unable to create Internet Gateway")
        else:
            igw_vpc_id = None

            if len(igw["attachments"]) > 0:
                igw_vpc_id = igw["attachments"][0]["vpc_id"]

                if detach_vpc:
                    if self._check_mode:
                        self._results["changed"] = True
                        self._results["gateway_id"] = igw["internet_gateway_id"]
                        return self._results

                    self.detach_vpc(igw["internet_gateway_id"], igw_vpc_id)

                elif igw_vpc_id != vpc_id:
                    if self._check_mode:
                        self._results["changed"] = True
                        self._results["gateway_id"] = igw["internet_gateway_id"]
                        return self._results

                    if force_attach:
                        self.get_matching_vpc(vpc_id)

                        self.detach_vpc(igw["internet_gateway_id"], igw_vpc_id)
                        self.attach_vpc(igw["internet_gateway_id"], vpc_id)
                    else:
                        self._module.fail_json(msg="VPC already attached, but does not match requested VPC.")

            elif vpc_id:
                if self._check_mode:
                    self._results["changed"] = True
                    self._results["gateway_id"] = igw["internet_gateway_id"]
                    return self._results

                self.get_matching_vpc(vpc_id)
                self.attach_vpc(igw["internet_gateway_id"], vpc_id)

        # Modify tags
        self._results["changed"] |= ensure_ec2_tags(
            self._connection,
            self._module,
            igw["internet_gateway_id"],
            resource_type="internet-gateway",
            tags=tags,
            purge_tags=purge_tags,
            retry_codes="InvalidInternetGatewayID.NotFound",
        )

        # Update igw
        igw = self.get_matching_igw(vpc_id, gateway_id=igw["internet_gateway_id"])
        igw_info = self.get_igw_info(igw, vpc_id)
        self._results.update(igw_info)

        return self._results


def main():
    argument_spec = dict(
        internet_gateway_id=dict(),
        vpc_id=dict(),
        state=dict(default="present", choices=["present", "absent"]),
        tags=dict(required=False, type="dict", aliases=["resource_tags"]),
        purge_tags=dict(default=True, type="bool"),
        force_attach=dict(default=False, type="bool"),
        detach_vpc=dict(default=False, type="bool"),
    )

    required_if = [
        ("force_attach", True, ("vpc_id",), False),
        ("state", "absent", ("internet_gateway_id", "vpc_id"), True),
        ("detach_vpc", True, ("internet_gateway_id", "vpc_id"), True),
    ]

    mutually_exclusive = [("force_attach", "detach_vpc")]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
    )

    results = dict(changed=False)
    igw_manager = AnsibleEc2Igw(module=module, results=results)
    igw_manager.process()

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Dag Wieers (@dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_label
short_description: Manage labels
description:
- Manage labels on Cisco ACI Multi-Site.
author:
- Dag Wieers (@dagwieers)
options:
  label:
    description:
    - The name of the label.
    type: str
    aliases: [ name ]
  type:
    description:
    - The type of the label.
    type: str
    choices: [ site ]
    default: site
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new label
  cisco.mso.mso_label:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    label: Belgium
    type: site
    state: present
  delegate_to: localhost

- name: Remove a label
  cisco.mso.mso_label:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    label: Belgium
    state: absent
  delegate_to: localhost

- name: Query a label
  cisco.mso.mso_label:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    label: Belgium
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all labels
  cisco.mso.mso_label:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        label=dict(type="str", aliases=["name"]),
        type=dict(type="str", default="site", choices=["site"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["label"]],
            ["state", "present", ["label"]],
        ],
    )

    label = module.params.get("label")
    label_type = module.params.get("type")
    state = module.params.get("state")

    mso = MSOModule(module)

    label_id = None
    path = "labels"

    # Query for existing object(s)
    if label:
        mso.existing = mso.get_obj(path, displayName=label)
        if mso.existing:
            label_id = mso.existing.get("id")
            # If we found an existing object, continue with it
            path = "labels/{id}".format(id=label_id)
    else:
        mso.existing = mso.query_objs(path)

    if state == "query":
        pass

    elif state == "absent":
        mso.previous = mso.existing
        if mso.existing:
            if module.check_mode:
                mso.existing = {}
            else:
                mso.existing = mso.request(path, method="DELETE")

    elif state == "present":
        mso.previous = mso.existing

        payload = dict(
            id=label_id,
            displayName=label,
            type=label_type,
        )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            if mso.check_changed():
                if module.check_mode:
                    mso.existing = mso.proposed
                else:
                    mso.existing = mso.request(path, method="PUT", data=mso.sent)
        else:
            if module.check_mode:
                mso.existing = mso.proposed
            else:
                mso.existing = mso.request(path, method="POST", data=mso.sent)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_wireless_rf_profiles
short_description: Resource module for networks _wireless _rfprofiles
description:
- Manage operations create, update and delete of the resource networks _wireless _rfprofiles.
- Creates new RF profile for this network.
- Delete a RF Profile.
- Updates specified RF profile for this network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  apBandSettings:
    description: Settings that will be enabled if selectionType is set to 'ap'.
    suboptions:
      bandOperationMode:
        description: Choice between 'dual', '2.4ghz' or '5ghz'. Defaults to dual.
        type: str
      bandSteeringEnabled:
        description: Steers client to most open band. Can be either true or false. Defaults
          to true.
        type: bool
    type: dict
  bandSelectionType:
    description: Band selection can be set to either 'ssid' or 'ap'. This param is required
      on creation.
    type: str
  clientBalancingEnabled:
    description: Steers client to best available access point. Can be either true or
      false. Defaults to true.
    type: bool
  fiveGhzSettings:
    description: Settings related to 5Ghz band.
    suboptions:
      channelWidth:
        description: Sets channel width (MHz) for 5Ghz band. Can be one of 'auto', '20',
          '40' or '80'. Defaults to auto.
        type: str
      maxPower:
        description: Sets max power (dBm) of 5Ghz band. Can be integer between 2 and
          30. Defaults to 30.
        type: int
      minBitrate:
        description: Sets min bitrate (Mbps) of 5Ghz band. Can be one of '6', '9', '12',
          '18', '24', '36', '48' or '54'. Defaults to 12.
        type: int
      minPower:
        description: Sets min power (dBm) of 5Ghz band. Can be integer between 2 and
          30. Defaults to 8.
        type: int
      rxsop:
        description: The RX-SOP level controls the sensitivity of the radio. It is strongly
          recommended to use RX-SOP only after consulting a wireless expert. RX-SOP
          can be configured in the range of -65 to -95 (dBm). A value of null will reset
          this to the default.
        type: int
      validAutoChannels:
        description: Sets valid auto channels for 5Ghz band. Can be one of '36', '40',
          '44', '48', '52', '56', '60', '64', '100', '104', '108', '112', '116', '120',
          '124', '128', '132', '136', '140', '144', '149', '153', '157', '161' or '165'.Defaults
          to 36, 40, 44, 48, 52, 56, 60, 64, 100, 104, 108, 112, 116, 120, 124, 128,
          132, 136, 140, 144, 149, 153, 157, 161, 165.
        elements: int
        type: list
    type: dict
  minBitrateType:
    description: Minimum bitrate can be set to either 'band' or 'ssid'. Defaults to
      band.
    type: str
  name:
    description: The name of the new profile. Must be unique. This param is required
      on creation.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  perSsidSettings:
    description: Per-SSID radio settings by number.
    suboptions:
      '0':
        description: Settings for SSID 0.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '1':
        description: Settings for SSID 1.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '10':
        description: Settings for SSID 10.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '11':
        description: Settings for SSID 11.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '12':
        description: Settings for SSID 12.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '13':
        description: Settings for SSID 13.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '14':
        description: Settings for SSID 14.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '2':
        description: Settings for SSID 2.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '3':
        description: Settings for SSID 3.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '4':
        description: Settings for SSID 4.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '5':
        description: Settings for SSID 5.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '6':
        description: Settings for SSID 6.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '7':
        description: Settings for SSID 7.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '8':
        description: Settings for SSID 8.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
      '9':
        description: Settings for SSID 9.
        suboptions:
          bandOperationMode:
            description: Choice between 'dual', '2.4ghz' or '5ghz'.
            type: str
          bandSteeringEnabled:
            description: Steers client to most open band between 2.4 GHz and 5 GHz.
              Can be either true or false.
            type: bool
          minBitrate:
            description: Sets min bitrate (Mbps) of this SSID. Can be one of '1', '2',
              '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'.
            type: float
        type: dict
    type: dict
  rfProfileId:
    description: RfProfileId path parameter. Rf profile ID.
    type: str
  sixGhzSettings:
    description: Settings related to 6Ghz band.
    suboptions:
      channelWidth:
        description: Sets channel width (MHz) for 6Ghz band. Can be one of '0', '20',
          '40', '80' or '160'. Defaults to 0.
        type: str
      maxPower:
        description: Sets max power (dBm) of 6Ghz band. Can be integer between 2 and
          30. Defaults to 30.
        type: int
      minBitrate:
        description: Sets min bitrate (Mbps) of 6Ghz band. Can be one of '6', '9', '12',
          '18', '24', '36', '48' or '54'. Defaults to 12.
        type: int
      minPower:
        description: Sets min power (dBm) of 6Ghz band. Can be integer between 2 and
          30. Defaults to 8.
        type: int
      rxsop:
        description: The RX-SOP level controls the sensitivity of the radio. It is strongly
          recommended to use RX-SOP only after consulting a wireless expert. RX-SOP
          can be configured in the range of -65 to -95 (dBm). A value of null will reset
          this to the default.
        type: int
      validAutoChannels:
        description: Sets valid auto channels for 6Ghz band. Can be one of '1', '5',
          '9', '13', '17', '21', '25', '29', '33', '37', '41', '45', '49', '53', '57',
          '61', '65', '69', '73', '77', '81', '85', '89', '93', '97', '101', '105',
          '109', '113', '117', '121', '125', '129', '133', '137', '141', '145', '149',
          '153', '157', '161', '165', '169', '173', '177', '181', '185', '189', '193',
          '197', '201', '205', '209', '213', '217', '221', '225', '229' or '233'.Defaults
          to 1, 5, 9, 13, 17, 21, 25, 29, 33, 37, 41, 45, 49, 53, 57, 61, 65, 69, 73,
          77, 81, 85, 89, 93, 97, 101, 105, 109, 113, 117, 121, 125, 129, 133, 137,
          141, 145, 149, 153, 157, 161, 165, 169, 173, 177, 181, 185, 189, 193, 197,
          201, 205, 209, 213, 217, 221, 225, 229, 233.
        elements: int
        type: list
    type: dict
  transmission:
    description: Settings related to radio transmission.
    suboptions:
      enabled:
        description: Toggle for radio transmission. When false, radios will not transmit
          at all.
        type: bool
    type: dict
  twoFourGhzSettings:
    description: Settings related to 2.4Ghz band.
    suboptions:
      axEnabled:
        description: Determines whether ax radio on 2.4Ghz band is on or off. Can be
          either true or false. If false, we highly recommend disabling band steering.
          Defaults to true.
        type: bool
      maxPower:
        description: Sets max power (dBm) of 2.4Ghz band. Can be integer between 2 and
          30. Defaults to 30.
        type: int
      minBitrate:
        description: Sets min bitrate (Mbps) of 2.4Ghz band. Can be one of '1', '2',
          '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54'. Defaults to 11.
        type: float
      minPower:
        description: Sets min power (dBm) of 2.4Ghz band. Can be integer between 2 and
          30. Defaults to 5.
        type: int
      rxsop:
        description: The RX-SOP level controls the sensitivity of the radio. It is strongly
          recommended to use RX-SOP only after consulting a wireless expert. RX-SOP
          can be configured in the range of -65 to -95 (dBm). A value of null will reset
          this to the default.
        type: int
      validAutoChannels:
        description: Sets valid auto channels for 2.4Ghz band. Can be one of '1', '6'
          or '11'. Defaults to 1, 6, 11.
        elements: int
        type: list
    type: dict
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for wireless createNetworkWirelessRfProfile
  description: Complete reference of the createNetworkWirelessRfProfile API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-wireless-rf-profile
- name: Cisco Meraki documentation for wireless deleteNetworkWirelessRfProfile
  description: Complete reference of the deleteNetworkWirelessRfProfile API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-wireless-rf-profile
- name: Cisco Meraki documentation for wireless updateNetworkWirelessRfProfile
  description: Complete reference of the updateNetworkWirelessRfProfile API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-rf-profile
notes:
  - SDK Method used are
    wireless.Wireless.create_network_wireless_rf_profile,
    wireless.Wireless.delete_network_wireless_rf_profile,
    wireless.Wireless.update_network_wireless_rf_profile,

  - Paths used are
    post /networks/{networkId}/wireless/rfProfiles,
    delete /networks/{networkId}/wireless/rfProfiles/{rfProfileId},
    put /networks/{networkId}/wireless/rfProfiles/{rfProfileId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_wireless_rf_profiles:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    apBandSettings:
      bandOperationMode: dual
      bandSteeringEnabled: true
    bandSelectionType: ap
    clientBalancingEnabled: true
    fiveGhzSettings:
      channelWidth: auto
      maxPower: 30
      minBitrate: 12
      minPower: 8
      rxsop: -95
      validAutoChannels:
      - 36
      - 40
      - 44
      - 48
      - 52
      - 56
      - 60
      - 64
      - 100
      - 104
      - 108
      - 112
      - 116
      - 120
      - 124
      - 128
      - 132
      - 136
      - 140
      - 144
      - 149
      - 153
      - 157
      - 161
      - 165
    minBitrateType: band
    name: Main Office
    networkId: string
    perSsidSettings:
      '0':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '1':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '10':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '11':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '12':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '13':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '14':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '2':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '3':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '4':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '5':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '6':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '7':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '8':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '9':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
    sixGhzSettings:
      channelWidth: auto
      maxPower: 30
      minBitrate: 12
      minPower: 8
      rxsop: -95
      validAutoChannels:
      - 1
      - 5
      - 9
      - 13
      - 17
      - 21
      - 25
      - 29
      - 33
      - 37
      - 41
      - 45
      - 49
      - 53
      - 57
      - 61
      - 65
      - 69
      - 73
      - 77
      - 81
      - 85
      - 89
      - 93
      - 97
      - 101
      - 105
      - 109
      - 113
      - 117
      - 121
      - 125
      - 129
      - 133
      - 137
      - 141
      - 145
      - 149
      - 153
      - 157
      - 161
      - 165
      - 169
      - 173
      - 177
      - 181
      - 185
      - 189
      - 193
      - 197
      - 201
      - 205
      - 209
      - 213
      - 217
      - 221
      - 225
      - 229
      - 233
    transmission:
      enabled: true
    twoFourGhzSettings:
      axEnabled: true
      maxPower: 30
      minBitrate: 11
      minPower: 5
      rxsop: -95
      validAutoChannels:
      - 1
      - 6
      - 11

- name: Update by id
  cisco.meraki.networks_wireless_rf_profiles:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    apBandSettings:
      bandOperationMode: dual
      bandSteeringEnabled: true
    bandSelectionType: ap
    clientBalancingEnabled: true
    fiveGhzSettings:
      channelWidth: auto
      maxPower: 30
      minBitrate: 12
      minPower: 8
      rxsop: -95
      validAutoChannels:
      - 36
      - 40
      - 44
      - 48
      - 52
      - 56
      - 60
      - 64
      - 100
      - 104
      - 108
      - 112
      - 116
      - 120
      - 124
      - 128
      - 132
      - 136
      - 140
      - 144
      - 149
      - 153
      - 157
      - 161
      - 165
    minBitrateType: band
    name: '1234'
    networkId: string
    perSsidSettings:
      '0':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '1':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '10':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '11':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '12':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '13':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '14':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '2':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '3':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '4':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '5':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '6':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '7':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '8':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
      '9':
        bandOperationMode: dual
        bandSteeringEnabled: true
        minBitrate: 11
    rfProfileId: string
    sixGhzSettings:
      channelWidth: auto
      maxPower: 30
      minBitrate: 12
      minPower: 8
      rxsop: -95
      validAutoChannels:
      - 1
      - 5
      - 9
      - 13
      - 17
      - 21
      - 25
      - 29
      - 33
      - 37
      - 41
      - 45
      - 49
      - 53
      - 57
      - 61
      - 65
      - 69
      - 73
      - 77
      - 81
      - 85
      - 89
      - 93
      - 97
      - 101
      - 105
      - 109
      - 113
      - 117
      - 121
      - 125
      - 129
      - 133
      - 137
      - 141
      - 145
      - 149
      - 153
      - 157
      - 161
      - 165
      - 169
      - 173
      - 177
      - 181
      - 185
      - 189
      - 193
      - 197
      - 201
      - 205
      - 209
      - 213
      - 217
      - 221
      - 225
      - 229
      - 233
    transmission:
      enabled: true
    twoFourGhzSettings:
      axEnabled: true
      maxPower: 30
      minBitrate: 11
      minPower: 5
      rxsop: -95
      validAutoChannels:
      - 1
      - 6
      - 11

- name: Delete by id
  cisco.meraki.networks_wireless_rf_profiles:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    networkId: string
    rfProfileId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "networkId": "string",
      "name": "string",
      "clientBalancingEnabled": true,
      "minBitrateType": "string",
      "bandSelectionType": "string",
      "apBandSettings": {
        "bandOperationMode": "string",
        "bandSteeringEnabled": true
      },
      "twoFourGhzSettings": {
        "maxPower": 0,
        "minPower": 0,
        "minBitrate": 0,
        "validAutoChannels": [
          0
        ],
        "axEnabled": true,
        "rxsop": 0
      },
      "fiveGhzSettings": {
        "maxPower": 0,
        "minPower": 0,
        "minBitrate": 0,
        "validAutoChannels": [
          0
        ],
        "channelWidth": "string",
        "rxsop": 0
      },
      "transmission": {
        "enabled": true
      },
      "perSsidSettings": {
        "0": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "1": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "2": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "3": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "4": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "5": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "6": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "7": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "8": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "9": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "10": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "11": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "12": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "13": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        },
        "14": {
          "name": "string",
          "minBitrate": 0,
          "bandOperationMode": "string",
          "bandSteeringEnabled": true
        }
      }
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options:
    meraki_api_key:
        description:
          - meraki_api_key (string), API key generated in dashboard; can also be set as an environment variable MERAKI_DASHBOARD_API_KEY
        type: str
        required: true
    meraki_base_url:
        description:
          - meraki_base_url (string), preceding all endpoint resources
        type: str
        default: https://api.meraki.com/api/v1
    meraki_single_request_timeout:
        description:
          - meraki_single_request_timeout (integer), maximum number of seconds for each API call
        type: int
        default: 60
    meraki_certificate_path:
        description:
          - meraki_certificate_path (string), path for TLS/SSL certificate verification if behind local proxy
        type: str
        default: ''
    meraki_requests_proxy:
        description:
          - meraki_requests_proxy (string), proxy server and port, if needed, for HTTPS
        type: str
        default: ''
    meraki_wait_on_rate_limit:
        description:
          - meraki_wait_on_rate_limit (boolean), retry if 429 rate limit error encountered?
        type: bool
        default: true
    meraki_nginx_429_retry_wait_time:
        description:
          - meraki_nginx_429_retry_wait_time (integer), Nginx 429 retry wait time
        type: int
        default: 60
    meraki_action_batch_retry_wait_time:
        description:
          - meraki_action_batch_retry_wait_time (integer), action batch concurrency error retry wait time
        type: int
        default: 60
    meraki_retry_4xx_error:
        description:
          - meraki_retry_4xx_error (boolean), retry if encountering other 4XX error (besides 429)?
        type: bool
        default: false
    meraki_retry_4xx_error_wait_time:
        description:
          - meraki_retry_4xx_error_wait_time (integer), other 4XX error retry wait time
        type: int
        default: 60
    meraki_maximum_retries:
        description:
          - meraki_maximum_retries (integer), retry up to this many times when encountering 429s or other server-side errors
        type: int
        default: 2
    meraki_output_log:
        description:
          - meraki_output_log (boolean), create an output log file?
        type: bool
        default: true
    meraki_log_file_prefix:
        description:
          - meraki_log_file_prefix (string), log file name appended with date and timestamp
        type: str
        default: meraki_api_
    meraki_log_path:
        description:
          - log_path (string), path to output log; by default, working directory of script if not specified
        type: str
        default: ''
    meraki_print_console:
        description:
          - meraki_print_console (boolean), print logging output to console?
        type: bool
        default: true
    meraki_suppress_logging:
        description:
          - meraki_suppress_logging (boolean), disable all logging? you're on your own then!
        type: bool
        default: false
    meraki_simulate:
        description:
          - meraki_simulate (boolean), simulate POST/PUT/DELETE calls to prevent changes?
        type: bool
        default: false
    meraki_be_geo_id:
        description:
          - meraki_be_geo_id (string), optional partner identifier for API usage tracking; can also be set as an environment variable BE_GEO_ID
        type: str
        default: ''
    meraki_caller:
        description:
          - meraki_caller (string), optional identifier for API usage tracking; can also be set as an environment variable MERAKI_PYTHON_SDK_CALLER
        type: str
        default: ''
    meraki_use_iterator_for_get_pages:
        description:
          - meraki_use_iterator_for_get_pages (boolean), list* methods will return an iterator with each object instead of a complete list with all items
        type: bool
        default: false
    meraki_inherit_logging_config:
        description:
          - meraki_inherit_logging_config (boolean), Inherits your own logger instance
        type: bool
        default: false
notes:
    - "Supports C(check_mode)"
    - "The plugin runs on the control node and does not use any ansible connection plugins, but instead the embedded connection manager from Cisco meraki SDK"
    - "The parameters starting with meraki_ are used by the Cisco meraki Python SDK to establish the connection"
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = """
module: ssm_inventory_info
version_added: 6.0.0
short_description: Get SSM inventory information for EC2 instance

description:
    - Gather SSM inventory for EC2 instance configured with SSM.

author: 'Aubin Bikouo (@abikouo)'

options:
  instance_id:
    description:
      - EC2 instance id.
    required: true
    type: str

extends_documentation_fragment:
- amazon.aws.common.modules
- amazon.aws.region.modules
- amazon.aws.boto3
"""

EXAMPLES = """
- name: Retrieve SSM inventory info for instance id 'i-012345678902'
  community.aws.ssm_inventory_info:
    instance_id: 'i-012345678902'
"""


RETURN = """
ssm_inventory:
    returned: on success
    description: >
        SSM inventory information.
    type: dict
    sample: {
        'agent_type': 'amazon-ssm-agent',
        'agent_version': '3.2.582.0',
        'computer_name': 'ip-172-31-44-166.ec2.internal',
        'instance_id': 'i-039eb9b1f55934ab6',
        'instance_status': 'Active',
        'ip_address': '172.31.44.166',
        'platform_name': 'Fedora Linux',
        'platform_type': 'Linux',
        'platform_version': '37',
        'resource_type': 'EC2Instance'
    }
"""


try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class SsmInventoryInfoFailure(Exception):
    def __init__(self, exc, msg):
        self.exc = exc
        self.msg = msg
        super().__init__(self)


def get_ssm_inventory(connection, filters):
    try:
        return connection.get_inventory(Filters=filters)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        raise SsmInventoryInfoFailure(exc=e, msg="get_ssm_inventory() failed.")


def execute_module(module, connection):
    instance_id = module.params.get("instance_id")
    try:
        filters = [{"Key": "AWS:InstanceInformation.InstanceId", "Values": [instance_id]}]

        response = get_ssm_inventory(connection, filters)
        entities = response.get("Entities", [])
        ssm_inventory = {}
        if entities:
            content = entities[0].get("Data", {}).get("AWS:InstanceInformation", {}).get("Content", [])
            if content:
                ssm_inventory = camel_dict_to_snake_dict(content[0])
        module.exit_json(changed=False, ssm_inventory=ssm_inventory)
    except SsmInventoryInfoFailure as e:
        module.fail_json_aws(exception=e.exc, msg=e.msg)


def main():
    argument_spec = dict(
        instance_id=dict(required=True, type="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    connection = module.client("ssm")
    execute_module(module, connection)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr_bgp_neighbor_address_family config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""


from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.bgp_neighbor_address_family import (
    Bgp_neighbor_address_familyTemplate,
)


class Bgp_neighbor_address_family(ResourceModule):
    """
    The iosxr_bgp_neighbor_address_family config class
    """

    def __init__(self, module):
        super(Bgp_neighbor_address_family, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="bgp_neighbor_address_family",
            tmplt=Bgp_neighbor_address_familyTemplate(),
        )
        self.parsers = [
            "router",
            "aigp",
            "allowas_in",
            "as_override",
            "bestpath_origin_as_allow_invalid",
            "capability_orf_prefix",
            "default_originate",
            "long_lived_graceful_restart_capable",
            "long_lived_graceful_restart_stale_time",
            "maximum_prefix",
            "multipath",
            "next_hop_self",
            "next_hop_unchanged",
            "optimal_route_reflection_group_name",
            "origin_as",
            "remove_private_AS",
            "route_reflector_client",
            "send_community_ebgp",
            "send_community_gshut_ebgp",
            "send_extended_community_ebgp",
            "send_multicast_attributes",
            "soft_reconfiguration",
            "weight",
            "site_of_origin",
            "validation",
            "route_policy.inbound",
            "route_policy.outbound",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """

        for entry in self.want, self.have:
            self._bgp_list_to_dict(entry)

        wantd = self.want
        haved = self.have
        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(self.have, self.want)

        # if state is deleted, empty out wantd and set haved to elements to delete
        if self.state == "deleted":
            if wantd:
                to_del = {
                    "neighbors": self._set_to_delete(haved, wantd),
                    "vrfs": {},
                }

                for k, hvrf in iteritems(haved.get("vrfs", {})):
                    wvrf = wantd.get("vrfs", {}).get(k, {})
                    to_del["vrfs"][k] = {
                        "neighbors": self._set_to_delete(hvrf, wvrf),
                        "vrf": k,
                    }
                haved.update(to_del)

            wantd = {"as_number": haved.get("as_number")}

        self._compare(want=wantd, have=haved)

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Bgp_global network resource.
        """

        self._compare_neighbors(want=want, have=have)
        self._compare_vrf(want=want, have=have)
        if self.commands and "router bgp" not in self.commands[0]:
            self.commands.insert(
                0,
                self._tmplt.render(
                    {"as_number": want["as_number"]},
                    "router",
                    False,
                ),
            )

    def _compare_neighbors(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Bgp_global neighbor resource.
        """
        want_nbr = want.get("neighbors", {})
        have_nbr = have.get("neighbors", {})
        for name, entry in iteritems(want_nbr):
            have = have_nbr.pop(name, {})
            begin = len(self.commands)
            self._compare_af(want=entry, have=have)
            if len(self.commands) != begin:
                self.commands.insert(begin, "neighbor {0}".format(name))

        # for deleted and overridden state
        if self.state != "replaced":
            for name, entry in iteritems(have_nbr):
                begin = len(self.commands)
                self._compare_af(want={}, have=entry)
                if len(self.commands) != begin:
                    self.commands.insert(begin, "neighbor {0}".format(name))

    def _compare_af(self, want, have):
        """Custom handling of afs option
        :params want: the want BGP dictionary
        :params have: the have BGP dictionary
        """
        wafs = want.get("address_family", {})
        hafs = have.get("address_family", {})
        for name, entry in iteritems(wafs):
            begin = len(self.commands)
            af_have = hafs.pop(name, {})
            self.compare(parsers=self.parsers, want=entry, have=af_have)
            if len(self.commands) != begin or (not af_have and entry):
                self.commands.insert(
                    begin,
                    self._tmplt.render(
                        {"afi": entry.get("afi"), "safi": entry.get("safi")},
                        "address_family",
                        False,
                    ),
                )

        for name, entry in iteritems(hafs):
            self.addcmd(
                {"afi": entry.get("afi"), "safi": entry.get("safi")},
                "address_family",
                True,
            )

    def _compare_vrf(self, want, have):
        """Custom handling of VRFs option
        :params want: the want BGP dictionary
        :params have: the have BGP dictionary
        """
        wvrfs = want.get("vrfs", {})
        hvrfs = have.get("vrfs", {})
        for name, entry in iteritems(wvrfs):
            begin = len(self.commands)
            vrf_have = hvrfs.pop(name, {})
            self._compare_neighbors(want=entry, have=vrf_have)
            if len(self.commands) != begin:
                self.commands.insert(begin, "vrf {0}".format(name))
        # for deleted and replaced state
        for name, entry in iteritems(hvrfs):
            begin = len(self.commands)
            self._compare_neighbors(want={}, have=entry)
            if len(self.commands) != begin:
                self.commands.insert(begin, "vrf {0}".format(name))

    def _bgp_list_to_dict(self, data):
        if "neighbors" in data:
            for nbr in data["neighbors"]:
                if "address_family" in nbr:
                    nbr["address_family"] = {
                        (x["afi"], x.get("safi")): x for x in nbr["address_family"]
                    }
            data["neighbors"] = {x["neighbor_address"]: x for x in data["neighbors"]}

        if "vrfs" in data:
            for vrf in data["vrfs"]:
                self._bgp_list_to_dict(vrf)
            data["vrfs"] = {x["vrf"]: x for x in data["vrfs"]}

    def _set_to_delete(self, haved, wantd):
        neighbors = {}
        h_nbrs = haved.get("neighbors", {})
        w_nbrs = wantd.get("neighbors", {})

        for k, h_nbr in iteritems(h_nbrs):
            w_nbr = w_nbrs.pop(k, {})
            if w_nbr:
                neighbors[k] = h_nbr
                afs_to_del = {}
                h_addrs = h_nbr.get("address_family", {})
                w_addrs = w_nbr.get("address_family", {})
                for af, h_addr in iteritems(h_addrs):
                    if af in w_addrs:
                        afs_to_del[af] = h_addr
                neighbors[k]["address_family"] = afs_to_del

        return neighbors



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_vrf_contract
short_description: Manage vrf contracts in schema templates
description:
- Manage vrf contracts in schema templates on Cisco ACI Multi-Site.
author:
- Cindy Zhao (@cizhao)
version_added: '0.0.8'
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template to change.
    type: str
    required: true
  vrf:
    description:
    - The name of the VRF.
    type: str
    required: true
  contract:
    description:
    - A contract associated to this VRF.
    type: dict
    suboptions:
      name:
        description:
        - The name of the Contract to associate with.
        required: true
        type: str
      schema:
        description:
        - The schema that defines the referenced contract.
        - If this parameter is unspecified, it defaults to the current schema.
        type: str
      template:
        description:
        - The template that defines the referenced contract.
        type: str
      type:
        description:
        - The type of contract.
        type: str
        required: true
        choices: [ consumer, provider ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_vrf
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a contract to a VRF
  cisco.mso.mso_schema_template_vrf_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    contract:
      name: Contract 1
      type: consumer
    state: present
  delegate_to: localhost

- name: Remove a Contract
  cisco.mso.mso_schema_template_vrf_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    contract:
      name: Contract 1
      type: consumer
    state: absent
  delegate_to: localhost

- name: Query a specific Contract
  cisco.mso.mso_schema_template_vrf_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    contract:
      name: Contract 1
      type: consumer
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Contracts
  cisco.mso.mso_schema_template_vrf_contract:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    vrf: VRF 1
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_contractref_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        vrf=dict(type="str", required=True),
        contract=dict(type="dict", options=mso_contractref_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["contract"]],
            ["state", "present", ["contract"]],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    vrf = module.params.get("vrf")
    contract = module.params.get("contract")
    if contract is not None and contract.get("template") is not None:
        contract["template"] = contract.get("template").replace(" ", "")
    state = module.params.get("state")

    mso = MSOModule(module)
    if contract:
        if contract.get("schema") is None:
            contract["schema"] = schema
        contract["schema_id"] = mso.lookup_schema(contract.get("schema"))
        if contract.get("template") is None:
            contract["template"] = template

    # Get schema
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get template
    templates = [t.get("name") for t in schema_obj.get("templates")]
    if template not in templates:
        mso.fail_json(msg="Provided template '{0}' does not exist. Existing templates: {1}".format(template, ", ".join(templates)))
    template_idx = templates.index(template)

    # Get VRF
    vrfs = [e.get("name") for e in schema_obj.get("templates")[template_idx]["vrfs"]]
    if vrf not in vrfs:
        mso.fail_json(msg="Provided vrf '{vrf}' does not exist. Existing vrfs: {vrfs}".format(vrf=vrf, vrfs=", ".join(vrfs)))
    vrf_idx = vrfs.index(vrf)
    vrf_obj = schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]

    if not vrf_obj.get("vzAnyEnabled"):
        mso.fail_json(msg="vzAny attribute on vrf '{0}' is disabled.".format(vrf))

    # Get Contract
    if contract:
        provider_contracts = [c.get("contractRef") for c in schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]["vzAnyProviderContracts"]]
        consumer_contracts = [c.get("contractRef") for c in schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]["vzAnyConsumerContracts"]]
        contract_ref = mso.contract_ref(**contract)
        if contract_ref in provider_contracts and contract.get("type") == "provider":
            contract_idx = provider_contracts.index(contract_ref)
            contract_path = "/templates/{0}/vrfs/{1}/vzAnyProviderContracts/{2}".format(template, vrf, contract_idx)
            mso.existing = schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]["vzAnyProviderContracts"][contract_idx]
        if contract_ref in consumer_contracts and contract.get("type") == "consumer":
            contract_idx = consumer_contracts.index(contract_ref)
            contract_path = "/templates/{0}/vrfs/{1}/vzAnyConsumerContracts/{2}".format(template, vrf, contract_idx)
            mso.existing = schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]["vzAnyConsumerContracts"][contract_idx]
        if mso.existing.get("contractRef"):
            mso.existing["contractRef"] = mso.dict_from_ref(mso.existing.get("contractRef"))
            mso.existing["relationshipType"] = contract.get("type")

    if state == "query":
        if not contract:
            provider_contracts = [
                dict(contractRef=mso.dict_from_ref(c.get("contractRef")), relationshipType="provider")
                for c in schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]["vzAnyProviderContracts"]
            ]
            consumer_contracts = [
                dict(contractRef=mso.dict_from_ref(c.get("contractRef")), relationshipType="consumer")
                for c in schema_obj.get("templates")[template_idx]["vrfs"][vrf_idx]["vzAnyConsumerContracts"]
            ]
            mso.existing = provider_contracts + consumer_contracts
        elif not mso.existing:
            mso.fail_json(msg="Contract '{0}' not found".format(contract.get("name")))

        mso.exit_json()

    if contract.get("type") == "provider":
        contracts_path = "/templates/{0}/vrfs/{1}/vzAnyProviderContracts/-".format(template, vrf)
    if contract.get("type") == "consumer":
        contracts_path = "/templates/{0}/vrfs/{1}/vzAnyConsumerContracts/-".format(template, vrf)
    ops = []

    mso.previous = mso.existing
    if state == "absent":
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op="remove", path=contract_path))

    elif state == "present":
        payload = dict(
            contractRef=dict(
                contractName=contract.get("name"),
                templateName=contract.get("template"),
                schemaId=contract.get("schema_id"),
            ),
        )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op="replace", path=contract_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=contracts_path, value=mso.sent))

        mso.existing = mso.proposed
        mso.existing["relationshipType"] = contract.get("type")

    if not module.check_mode and mso.proposed != mso.previous:
        mso.request(schema_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2022, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: openssl_privatekey_convert
short_description: Convert OpenSSL private keys
version_added: 2.1.0
description:
    - This module allows one to convert OpenSSL private keys.
    - The default mode for the private key file will be V(0600) if O(mode) is not explicitly set.
author:
    - Felix Fontein (@felixfontein)
extends_documentation_fragment:
    - ansible.builtin.files
    - community.crypto.attributes
    - community.crypto.attributes.files
    - community.crypto.module_privatekey_convert
attributes:
    check_mode:
        support: full
    diff_mode:
        support: none
    safe_file_operations:
        support: full
options:
    dest_path:
        description:
            - Name of the file in which the generated TLS/SSL private key will be written. It will have V(0600) mode
              if O(mode) is not explicitly set.
        type: path
        required: true
    backup:
        description:
            - Create a backup file including a timestamp so you can get
              the original private key back if you overwrote it with a new one by accident.
        type: bool
        default: false
seealso: []
'''

EXAMPLES = r'''
- name: Convert private key to PKCS8 format with passphrase
  community.crypto.openssl_privatekey_convert:
    src_path: /etc/ssl/private/ansible.com.pem
    dest_path: /etc/ssl/private/ansible.com.key
    dest_passphrase: '{{ private_key_passphrase }}'
    format: pkcs8
'''

RETURN = r'''
backup_file:
    description: Name of backup file created.
    returned: changed and if O(backup) is V(true)
    type: str
    sample: /path/to/privatekey.pem.2019-03-09@11:22~
'''

import os

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.crypto.plugins.module_utils.io import (
    load_file_if_exists,
    write_file,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.basic import (
    OpenSSLObjectError,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.support import (
    OpenSSLObject,
)

from ansible_collections.community.crypto.plugins.module_utils.crypto.module_backends.privatekey_convert import (
    select_backend,
    get_privatekey_argument_spec,
)


class PrivateKeyConvertModule(OpenSSLObject):
    def __init__(self, module, module_backend):
        super(PrivateKeyConvertModule, self).__init__(
            module.params['dest_path'],
            'present',
            False,
            module.check_mode,
        )
        self.module_backend = module_backend

        self.backup = module.params['backup']
        self.backup_file = None

        module.params['path'] = module.params['dest_path']
        if module.params['mode'] is None:
            module.params['mode'] = '0600'

        module_backend.set_existing_destination(load_file_if_exists(self.path, module))

    def generate(self, module):
        """Do conversion."""

        if self.module_backend.needs_conversion():
            # Convert
            privatekey_data = self.module_backend.get_private_key_data()
            if not self.check_mode:
                if self.backup:
                    self.backup_file = module.backup_local(self.path)
                write_file(module, privatekey_data, 0o600)
            self.changed = True

        file_args = module.load_file_common_arguments(module.params)
        if module.check_file_absent_if_check_mode(file_args['path']):
            self.changed = True
        else:
            self.changed = module.set_fs_attributes_if_different(file_args, self.changed)

    def dump(self):
        """Serialize the object into a dictionary."""

        result = self.module_backend.dump()
        result['changed'] = self.changed
        if self.backup_file:
            result['backup_file'] = self.backup_file

        return result


def main():

    argument_spec = get_privatekey_argument_spec()
    argument_spec.argument_spec.update(dict(
        dest_path=dict(type='path', required=True),
        backup=dict(type='bool', default=False),
    ))
    module = argument_spec.create_ansible_module(
        supports_check_mode=True,
        add_file_common_args=True,
    )

    base_dir = os.path.dirname(module.params['dest_path']) or '.'
    if not os.path.isdir(base_dir):
        module.fail_json(
            name=base_dir,
            msg='The directory %s does not exist or the file is not a directory' % base_dir
        )

    module_backend = select_backend(module=module)

    try:
        private_key = PrivateKeyConvertModule(module, module_backend)

        private_key.generate(module)

        result = private_key.dump()
        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=to_native(exc))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type
try:
    from ciscoisesdk import api, exceptions
except ImportError:
    ISE_SDK_IS_INSTALLED = False
else:
    ISE_SDK_IS_INSTALLED = True
from ansible.module_utils.basic import env_fallback

try:
    from ansible.errors import AnsibleActionFail
except ImportError:
    ANSIBLE_ERRORS_INSTALLED = False
else:
    ANSIBLE_ERRORS_INSTALLED = True
try:
    import logging
except ImportError:
    LOGGING_IN_STANDARD = False
else:
    LOGGING_IN_STANDARD = True


def is_list_complex(x):
    return isinstance(x[0], dict) or isinstance(x[0], list)


def has_diff_elem(ls1, ls2):
    return any((elem not in ls1 for elem in ls2))


def compare_list(list1, list2):
    len_list1 = len(list1)
    len_list2 = len(list2)
    if len_list1 != len_list2:
        return False

    if len_list1 == 0:
        return True

    attempt_std_cmp = list1 == list2
    if attempt_std_cmp:
        return True

    if not is_list_complex(list1) and not is_list_complex(list2):
        return set(list1) == set(list2)

    # Compare normally if it exceeds expected size * 2 (len_list1==len_list2)
    MAX_SIZE_CMP = 100
    # Fail fast if elem not in list, thanks to any and generators
    if len_list1 > MAX_SIZE_CMP:
        return attempt_std_cmp
    else:
        # not changes 'has diff elem' to list1 != list2 ':lists are not equal'
        has_diff_1 = has_diff_elem(list1, list2)
        has_diff_2 = has_diff_elem(list2, list1)
        return not has_diff_1 or not has_diff_2


def fn_comp_key(k, dict1, dict2):
    return ise_compare_equality(dict1.get(k), dict2.get(k))


def ise_compare_equality(current_value, requested_value):
    if requested_value is None:
        return True
    if current_value is None:
        return True
    if isinstance(current_value, dict) and isinstance(requested_value, dict):
        all_dict_params = list(current_value.keys()) + list(requested_value.keys())
        return not any((not fn_comp_key(param, current_value, requested_value) for param in all_dict_params))
    elif isinstance(current_value, list) and isinstance(requested_value, list):
        return compare_list(current_value, requested_value)
    else:
        return current_value == requested_value


def fn_comp_key2(k, dict1, dict2):
    return ise_compare_equality2(dict1.get(k), dict2.get(k))


def ise_compare_equality2(current_value, requested_value, is_query_param=False):
    if is_query_param:
        return True
    if requested_value is None and current_value is None:
        return True
    if requested_value is None:
        return False
    if current_value is None:
        return False
    if isinstance(current_value, dict) and isinstance(requested_value, dict):
        all_dict_params = list(current_value.keys()) + list(requested_value.keys())
        return not any((not fn_comp_key2(param, current_value, requested_value) for param in all_dict_params))
    elif isinstance(current_value, list) and isinstance(requested_value, list):
        return compare_list(current_value, requested_value)
    else:
        return current_value == requested_value


def get_dict_result(result, key, value):
    if isinstance(result, list):
        if len(result) == 1:
            if isinstance(result[0], dict):
                result = result[0]
                if result.get(key) is not None and result.get(key) != value:
                    result = None
            else:
                result = None
        else:
            for item in result:
                if isinstance(item, dict) and (item.get(key) is None or item.get(key) == value):
                    result = item
                    return result
            result = None
    elif not isinstance(result, dict):
        result = None
    elif result.get(key) is not None and result.get(key) != value:
        result = None
    return result


def ise_argument_spec():
    argument_spec = dict(
        ise_hostname=dict(type="str", fallback=(env_fallback, ['ISE_HOSTNAME']), required=True),
        ise_username=dict(type="str", fallback=(env_fallback, ['ISE_USERNAME']), required=True),
        ise_password=dict(type="str", fallback=(env_fallback, ['ISE_PASSWORD']), required=True, no_log=True),
        ise_verify=dict(type="bool", default=True, fallback=(env_fallback, ['ISE_VERIFY'])),
        ise_version=dict(type="str", default="3.2_beta", fallback=(env_fallback, ['ISE_VERSION'])),
        ise_wait_on_rate_limit=dict(type="bool", default=True, fallback=(env_fallback, ['ISE_WAIT_ON_RATE_LIMIT'])),
        ise_uses_api_gateway=dict(type="bool", default=True, fallback=(env_fallback, ['ISE_USES_API_GATEWAY'])),
        ise_uses_csrf_token=dict(type="bool", default=False, fallback=(env_fallback, ['ISE_USES_CSRF_TOKEN'])),
        ise_debug=dict(type="bool", default=False, fallback=(env_fallback, ['ISE_DEBUG'])),
        ise_single_request_timeout=dict(type="int", default=60, fallback=(env_fallback, ['ISE_SINGLE_REQUEST_TIMEOUT'])),
    )
    return argument_spec


def get_ise_url(hostname, port=None):
    url_result = "https://{hostname}".format(hostname=hostname)
    if port:
        url_result = url_result + ":{port}".format(port=port)
    return url_result


class ISESDK(object):
    def __init__(self, params):
        self.result = dict(changed=False, result="")
        if ISE_SDK_IS_INSTALLED:
            ise_uses_api_gateway = params.get("ise_uses_api_gateway")
            ui_base_url = None
            ers_base_url = None
            mnt_base_url = None
            px_grid_base_url = None
            if not ise_uses_api_gateway:
                ui_base_url = get_ise_url(params.get("ise_hostname"), port="443")
                ers_base_url = get_ise_url(params.get("ise_hostname"), port="9060")
                mnt_base_url = get_ise_url(params.get("ise_hostname"), port="443")
                px_grid_base_url = get_ise_url(params.get("ise_hostname"), port="8910")

            self.api = api.IdentityServicesEngineAPI(
                username=params.get("ise_username"),
                password=params.get("ise_password"),
                base_url=get_ise_url(params.get("ise_hostname"), port=None),
                ui_base_url=ui_base_url,
                ers_base_url=ers_base_url,
                mnt_base_url=mnt_base_url,
                px_grid_base_url=px_grid_base_url,
                single_request_timeout=params.get("ise_single_request_timeout"),
                verify=params.get("ise_verify"),
                version=params.get("ise_version"),
                wait_on_rate_limit=params.get("ise_wait_on_rate_limit"),
                uses_api_gateway=ise_uses_api_gateway,
                uses_csrf_token=params.get("ise_uses_csrf_token"),
                debug=params.get("ise_debug"),
            )
            if params.get("ise_debug") and LOGGING_IN_STANDARD:
                logging.getLogger('ciscoisesdk').addHandler(logging.StreamHandler())
        else:
            self.fail_json(msg="Cisco ISE Python SDK is not installed. Execute 'pip install ciscoisesdk'")

    def changed(self):
        self.result["changed"] = True

    def object_created(self):
        self.changed()
        self.result["result"] = "Object created"

    def object_updated(self):
        self.changed()
        self.result["result"] = "Object updated"

    def object_deleted(self):
        self.changed()
        self.result["result"] = "Object deleted"

    def object_already_absent(self):
        self.result["result"] = "Object already absent"

    def object_already_present(self):
        self.result["result"] = "Object already present"

    def object_present_and_different(self):
        self.result["result"] = "Object already present, but it has different values to the requested"

    def object_modify_result(self, changed=None, result=None):
        if result is not None:
            self.result["result"] = result
        if changed:
            self.changed()

    def exec(self, family, function, params=None, handle_func_exception=True):
        try:
            family = getattr(self.api, family)
            func = getattr(family, function)
        except Exception as e:
            self.fail_json(
                msg=(
                    "An error occured when retrieving operation."
                    " The error was: {error}"
                ).format(error=e)
            )

        try:
            if params:
                response = func(**params)
            else:
                response = func()
        except exceptions.ciscoisesdkException as e:
            if handle_func_exception:
                self.fail_json(
                    msg=(
                        "An error occured when executing operation."
                        " The error was: {error}"
                    ).format(error=e)
                )
            else:
                raise e
        return response

    def fail_json(self, msg, **kwargs):
        self.result.update(**kwargs)
        raise AnsibleActionFail(msg, kwargs)

    def exit_json(self):
        return self.result


def main():
    pass


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright (c) 2018, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import traceback

from ansible.module_utils.basic import env_fallback, missing_required_lib

HAS_HEROKU = False
HEROKU_IMP_ERR = None
try:
    import heroku3
    HAS_HEROKU = True
except ImportError:
    HEROKU_IMP_ERR = traceback.format_exc()


class HerokuHelper():
    def __init__(self, module):
        self.module = module
        self.check_lib()
        self.api_key = module.params["api_key"]

    def check_lib(self):
        if not HAS_HEROKU:
            self.module.fail_json(msg=missing_required_lib('heroku3'), exception=HEROKU_IMP_ERR)

    @staticmethod
    def heroku_argument_spec():
        return dict(
            api_key=dict(fallback=(env_fallback, ['HEROKU_API_KEY', 'TF_VAR_HEROKU_API_KEY']), type='str', no_log=True))

    def get_heroku_client(self):
        client = heroku3.from_key(self.api_key)

        if not client.is_authenticated:
            self.module.fail_json(msg='Heroku authentication failure, please check your API Key')

        return client



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: device_replacement
short_description: Resource module for Device Replacement
description:
- Manage operations create and update of the resource Device Replacement.
- Marks device for replacement.
- UnMarks device for replacement.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Device Replacement's payload.
    elements: dict
    suboptions:
      creationTime:
        description: Device Replacement's creationTime.
        type: int
      family:
        description: Device Replacement's family.
        type: str
      faultyDeviceId:
        description: Device Replacement's faultyDeviceId.
        type: str
      faultyDeviceName:
        description: Device Replacement's faultyDeviceName.
        type: str
      faultyDevicePlatform:
        description: Device Replacement's faultyDevicePlatform.
        type: str
      faultyDeviceSerialNumber:
        description: Device Replacement's faultyDeviceSerialNumber.
        type: str
      id:
        description: Device Replacement's id.
        type: str
      neighbourDeviceId:
        description: Device Replacement's neighbourDeviceId.
        type: str
      networkReadinessTaskId:
        description: Device Replacement's networkReadinessTaskId.
        type: str
      replacementDevicePlatform:
        description: Device Replacement's replacementDevicePlatform.
        type: str
      replacementDeviceSerialNumber:
        description: Device Replacement's replacementDeviceSerialNumber.
        type: str
      replacementStatus:
        description: Device Replacement's replacementStatus.
        type: str
      replacementTime:
        description: Device Replacement's replacementTime.
        type: int
      workflowId:
        description: Device Replacement's workflowId.
        type: str
    type: list
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Replacement MarkDeviceForReplacement
  description: Complete reference of the MarkDeviceForReplacement API.
  link: https://developer.cisco.com/docs/dna-center/#!mark-device-for-replacement
- name: Cisco DNA Center documentation for Device Replacement UnMarkDeviceForReplacement
  description: Complete reference of the UnMarkDeviceForReplacement API.
  link: https://developer.cisco.com/docs/dna-center/#!un-mark-device-for-replacement
notes:
  - SDK Method used are
    device_replacement.DeviceReplacement.mark_device_for_replacement,
    device_replacement.DeviceReplacement.unmark_device_for_replacement,

  - Paths used are
    post /dna/intent/api/v1/device-replacement,
    put /dna/intent/api/v1/device-replacement,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.device_replacement:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - creationTime: 0
      family: string
      faultyDeviceId: string
      faultyDeviceName: string
      faultyDevicePlatform: string
      faultyDeviceSerialNumber: string
      id: string
      neighbourDeviceId: string
      networkReadinessTaskId: string
      replacementDevicePlatform: string
      replacementDeviceSerialNumber: string
      replacementStatus: string
      replacementTime: 0
      workflowId: string

- name: Create
  cisco.dnac.device_replacement:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - creationTime: 0
      family: string
      faultyDeviceId: string
      faultyDeviceName: string
      faultyDevicePlatform: string
      faultyDeviceSerialNumber: string
      id: string
      neighbourDeviceId: string
      networkReadinessTaskId: string
      replacementDevicePlatform: string
      replacementDeviceSerialNumber: string
      replacementStatus: string
      replacementTime: 0
      workflowId: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""
